@@ -880,14 +880,16 @@ class AzureML(BackendBase):
                 if container_registry:
                     environment.docker.base_image_registry = container_registry
 
-                # let user specify additional docker options
-                other_args = args["docker_other_options"] if args["docker_other_options"] else []
+                # singularity backend doesn't support other docker commands
+                if self.get_name() != "singularity":
+                    # let user specify additional docker options
+                    other_args = args["docker_other_options"] if args["docker_other_options"] else []
 
-                if use_sudo:
-                    # specify additional docker run flags so that we can use blobfuse
-                    other_args += ["--privileged"]
+                    if use_sudo:
+                        # specify additional docker run flags so that we can use blobfuse
+                        other_args += ["--privileged"]
 
-                environment.docker.arguments = other_args
+                    environment.docker.arguments = other_args
 
                 if docker_image:
                     # override AML default base image with user-specified image
