@@ -193,11 +193,30 @@ class PoolBackend(BackendBase):
                 node_id = "node" + str(i)
                 info_by_node[node_id] = service_node_info
 
+                # update node_info record in database with service_info
+                db = self.client.store.get_database()
+                db.update_node_info_with_service_info(workspace, job_id, i, service_node_info)
+
         return job_info, info_by_node
 
     def run_job_on_box(self, job_id, run_data_list, box_index, box_info, app_info, pool_info,  
             resume_name=None, repeat=None, using_hp=None, exper_name=None, snapshot_dir=None, args=None):
 
+        '''
+        Remote box run - order of operations:
+            1. psm_client = RemotePsmClient()
+
+            2. psm_client runs locally and starts SSH with remote box
+
+            3. psm_client.restart_psm_if_needed()
+                - kills psm on remote if using old psm
+                - copies psm.py to remote box
+                - starts running psm.py on remote box
+            
+            4. psm_client.enqueue()
+                - adds .zip file for run to node's queue 
+        '''
+
         box_name = box_info.box_name
         box_addr = box_info.address
         box_os = box_info.box_os
@@ -224,7 +243,8 @@ class PoolBackend(BackendBase):
         fn_entry = psm_client.enqueue(store_name, workspace, job_id, run_name, box_index, fn_src_zip)
 
         service_node_info = {"fn_entry": fn_entry, "box_addr": box_addr, "box_os": box_os, 
-            "box_name": box_name, "job_id": job_id, "run_name": run_name}
+            "box_name": box_name, "job_id": job_id, "run_name": run_name, 
+            "node_id": "node"+str(box_index), "workspace": workspace}
 
         fb.feedback("submitted") 
 
@@ -401,34 +421,56 @@ class PoolBackend(BackendBase):
         job_id = service_node_info["job_id"]
         fn_entry = service_node_info["fn_entry"]
 
+        # safe values for newly-added fields
+        node_id = utils.safe_value(service_node_info, "node_id", "node0")
+        workspace = utils.safe_value(service_node_info, "workspace", "<unavailable>")
+
         psm_client = self.get_psm_client(service_node_info)
 
         next_offset = 0
         new_text = None
+        found_file = False
 
-        # read LOG FILE
-        #start = time.time()
-        new_bytes = psm_client.read_log_file(fn_entry, start_offset, end_offset)
-        next_offset = start_offset + len(new_bytes)
-        #utils.report_elapsed(start, "read log")
+        if log_name is None:
+            log_name = "stdboth.txt"
+        file_path = os.path.basename(log_name)
 
-        new_text = new_bytes.decode(encoding)
-        service_status = None
+        # try to first read from job storage (if task has completed)
+        if log_source != "live":
+            node_index = utils.node_index(node_id)
 
-        if new_text:
-            service_status = "running"
-        else:
-            # refresh STATUS 
-            # we retry "completed" a few times to avoid a false positive when transitioning between the 
-            # wrapper script and the controller app
-            count = 10 if start_offset == 0 else 3
+            job_path = "nodes/node{}/after/service_logs/{}".format(node_index, file_path)
+            if self.core.store.does_job_file_exist(workspace, job_id, job_path):
+                new_text = self.core.store.read_job_file(workspace, job_id, job_path)
+                service_status = "completed"
+                simple_status = "completed"
+                found_file = True
+                log_source = "after_logs"
 
-            for _ in range(count):
-                service_status = psm_client.get_status(fn_entry)
-                if service_status != "completed":
-                    break
+        if not found_file:
+            # read LOG FILE
+            #start = time.time()
+            new_bytes = psm_client.read_log_file(fn_entry, start_offset, end_offset)
+            next_offset = start_offset + len(new_bytes)
+            #utils.report_elapsed(start, "read log")
 
-        simple_status = self.get_simple_status(service_status)
+            new_text = new_bytes.decode(encoding)
+            service_status = None
+
+            if new_text:
+                service_status = "running"
+            else:
+                # refresh STATUS 
+                # we retry "completed" a few times to avoid a false positive when transitioning between the 
+                # wrapper script and the controller app
+                count = 10 if start_offset == 0 else 3
+
+                for _ in range(count):
+                    service_status = psm_client.get_status(fn_entry)
+                    if service_status != "completed":
+                        break
+
+            simple_status = self.get_simple_status(service_status)
 
         return {"new_text": new_text, "simple_status": simple_status, "log_name": log_name, "next_offset": next_offset, 
             "service_status": service_status}
@@ -515,19 +557,21 @@ class PoolBackend(BackendBase):
 
         return job_records
 
-    def get_log_save_cmd(self, output_dir):
-
-        # 3rd argument to script is the path of entry's log file
-        if self.gen_for_windows:
-            cmd = "cp -v %3 {}".format(output_dir)
-        else:
-            cmd = "cp -v $3 {}".format(output_dir)
-        return cmd
-
    # API call
     def add_service_log_copy_cmds(self, cmds, dest_dir, args):
-        # $3 is the path to the local log file for the node
+        # if using docker, must use /usr/src/logs for log dir
+        docker_cmd = args["docker_cmd"]
+        
         if self.gen_for_windows:
-            self.append(cmds, "xcopy %3 {}\\".format(dest_dir))
+            self.append(cmds, "xcopy logs\* {}\\".format(dest_dir))
         else:
-            self.append(cmds, "mkdir {} && cp $3 {}".format(dest_dir, dest_dir))
+            if docker_cmd:
+                self.append(cmds, "mkdir {} && cp /usr/src/logs/* {}".format(dest_dir, dest_dir))
+            else:
+                self.append(cmds, "mkdir {} && cp logs/* {}".format(dest_dir, dest_dir))
+    # API call
+    def get_log_files_dir(self, args):
+        docker_cmd = args["docker_cmd"]
+
+        path = "/usr/src/logs" if docker_cmd else "logs"
+        return path
