@@ -26,6 +26,24 @@ def calc_size_of_files(filenames):
 
     return total
 
+def zip_before_files(snapshot_dir, omit_files, zip_type, remove_prefix_len=0):
+
+    filenames = file_helper.get_filenames_from_include_lists([snapshot_dir], omit_files, recursive=True)
+    count = len(filenames)
+
+    size = calc_size_of_files(filenames)
+    unzipped_mb = size/(1024*1024)
+    zip_name = constants.CODE_ZIP_FN 
+
+    fn_zip = os.path.expanduser(constants.CWD_DIR + "/" + zip_name)
+    use_compress = (zip_type == "compress")
+    file_helper.zip_up_filenames(fn_zip, filenames, use_compress, remove_prefix_len)
+
+    size = calc_size_of_files([fn_zip])
+    zipped_mb = size/(1024*1024)
+
+    return fn_zip, count, unzipped_mb, zipped_mb
+
 def capture_before_files_zip(store, source_dir, ws_name=None, run_name=None, extra_files = [], rerun_name=None, 
         job_id=None,  omit_files=None, zip_before="none", store_dest="before", remove_prefix_len=None, 
         upload_type=None, service_type=None, args=None):
@@ -120,29 +138,33 @@ def capture_before_files_zip(store, source_dir, ws_name=None, run_name=None, ext
 
     return copied_files
 
-def download_before_files(store, job_id, ws_name, run_name, dest_dir, silent=False, log_events=True, source="job"):
+def download_before_files(store, job_id, ws_name, run_name, dest_dir, silent=False, log_events=True, source="job", 
+    unzip=True, source_wildcard=None):
+
     files = []
+    if not source_wildcard:
+        source_wildcard = "before/**"
 
     if source == "job":
         # all files contained in JOB BEFORE
-        files += store.download_files_from_job(ws_name, job_id, "before/code/**", dest_dir)
+        files += store.download_files_from_job(ws_name, job_id, source_wildcard, dest_dir)
         count = len(files)
         if not silent:
             console.print("  {} CODE files downloaded from store (JOB dir)".format(count))
 
-        if log_events:
+        if log_events and run_name:
             store.log_run_event(ws_name, run_name, "download_before", {"source": "job", "count": count}, job_id=job_id)
     else:
         if "." in run_name:
             # download from parent's RUN before files
             parent_name = run_name.split(".")[0]
-            files += store.download_files_from_run(ws_name, parent_name, "before/**", dest_dir)
+            files += store.download_files_from_run(ws_name, parent_name, source_wildcard, dest_dir)
             count = len(files)
             if not silent:
                 console.print("  {} files downloaded from store (PARENT'S RUN dir)".format(count))
 
             if log_events:
-                store.log_run_event(ws_name, run_name, "download_before", {"source": "parent-run", "count": count})
+                store.log_run_event(ws_name, run_name, source_wildcard, {"source": "parent-run", "count": count})
         
         # download from normal/child RUN before files
         files += store.download_files_from_run(ws_name, run_name, "before/**", dest_dir, job_id=job_id)
@@ -153,7 +175,8 @@ def download_before_files(store, job_id, ws_name, run_name, dest_dir, silent=Fal
         if log_events:
             store.log_run_event(ws_name, run_name, "download_before", {"source": "run", "count": count})
 
-    unzip_before_if_found(dest_dir, silent=silent)
+    if unzip:
+        unzip_before_if_found(dest_dir, silent=silent)
 
     return files
 
