@@ -3,14 +3,12 @@
 # Licensed under the MIT license.
 #
 # controller.py - running on the compute box, controls the management of all XT or XTLib initiated jobs for that machine.
-import sys
 import os
+import sys
 import json
 import rpyc
 import copy
 import time
-import shutil 
-import random
 import psutil
 import logging
 import datetime
@@ -819,6 +817,41 @@ class XTController(rpyc.Service):
             log_info("xt controller", "scheduling shutdown...")
             self.shutdown_requested = True
 
+    def get_cmdline_args(self, cmd_parts, option_prefix):
+        arg_dict = {}
+        pending_name = None
+        #print("CONTROLLER: cmd_parts=", cmd_parts)
+
+        for part in cmd_parts:
+            if option_prefix and part.startswith(option_prefix):
+                # start of new option being specified
+
+                if pending_name:
+                    # assume previous option name is a flag
+                    arg_dict[pending_name] = 1
+                    pending_name = None
+
+                part = part[len(option_prefix):]
+                pending_name = None
+
+                if "=" in part:
+                    name, value = part.split("=")
+                    arg_dict[name] = value
+                else:
+                    # name specified but not value
+                    pending_name = part
+
+            elif pending_name:
+                # found value for pending name
+                arg_dict[pending_name] = part
+                pending_name = None
+
+        if pending_name:
+            # assume previous option name is a flag
+            arg_dict[pending_name] = 1
+        
+        return arg_dict
+
     def spawn_child(self, parent, run_index, entry):
         spawn_start = time.time()
 
@@ -882,7 +915,8 @@ class XTController(rpyc.Service):
                 # select the normal command for the run
                 log_info("normal child without HPARAM search", child_name)
                 cmd_parts = context.cmd_parts
-                hp_set = None
+                hp_dict = self.get_cmdline_args(cmd_parts, context.option_prefix)
+                hp_set = str(hp_dict)
 
         if cmd_parts:
             # HP search succeeded
@@ -905,7 +939,8 @@ class XTController(rpyc.Service):
                 job_id=context.job_id, pool=context.pool, node_index=self.node_index, 
                 aggregate_dest=context.aggregate_dest,  
                 compute=context.compute, service_type=context.service_type, username=context.username, 
-                search_type=search_type, run_index=run_index, hp_set=hp_set, display_name=display_name)
+                search_type=search_type, run_index=run_index, hp_set=hp_set, display_name=display_name, 
+                cmd_line_args=str(cmd_parts), xt_cmd=context.xt_cmd)
 
             # must update context info
             context.run_name = child_name
@@ -1018,7 +1053,9 @@ class XTController(rpyc.Service):
     def fix_path(self, path):
         path = os.path.expanduser(path)
 
-        if  pc_utils.is_windows():
+        if pc_utils.is_windows():
+            console.print("FIXING UP path for WINDOWS: {}".format(path))
+
             path = path.replace("$HOME", "%USERPROFILE%")
             path = path.replace("/", "\\")
 
