@@ -57,16 +57,23 @@ def zap_dir(dir):
     ''' this will delete all files recursively, including readonly files (unlike shutil.rmtree)
     '''
     if os.path.exists(dir):
-        for name in os.listdir(dir):
-            path = os.path.join(dir, name)
 
-            if os.path.isfile(path):
-                zap_file(path)
-            else:
-                zap_dir(path)
+        if os.name != "nt":
+            # on linux, we call rm -rf to be sure hidden files and other issues are covered
+            cmd = "rm -rf {}".format(dir)
+            os.system(cmd)
+            
+        else:    
+            for name in os.listdir(dir):
+                path = os.path.join(dir, name)
 
-        # remove the root dir for our call
-        os.rmdir(dir)
+                if os.path.isfile(path):
+                    zap_file(path)
+                else:
+                    zap_dir(path)
+
+            # remove the root dir for our call
+            os.rmdir(dir)
 
 def ensure_dir_exists(dir=None, file=None):
     if file:
