@@ -104,7 +104,7 @@ class Validator():
 
             elif s_value == "$bool":
                 if not isinstance(c_value, bool):
-                    if not c_value in ["true", "false"]:
+                    if not c_value in ["true", "false", None]:
                         self.value_mismatch("bool", c_value, ck_path)
 
             elif s_value == "$int":
