@@ -397,21 +397,26 @@ class XTConfig():
             docker_name = utils.safe_value(compute_def, "docker")
             
         if docker_name:
-            env = self.get("dockers", docker_name, default_value=None)
-            if not env:
-                errors.config_error("docker '{}' definition not found in XT config file".format(docker_name))
+            if "@" in docker_name:
+                # special way of dynamically specifying the docker image name (without it being present in config file)
+                registry_name, docker_image = docker_name.split("@")
 
-            docker_image = utils.safe_value(env, "image")
-            if not docker_image:
-                errors.config_error("Definition for docker '{}' is missing the required 'image' property".format(docker_name))
+            else:
+                env = self.get("dockers", docker_name, default_value=None)
+                if not env:
+                    errors.config_error("docker '{}' definition not found in XT config file".format(docker_name))
 
-            sing_dict["sing_wrap"] = utils.safe_value(env, "sing-wrap")
-            sing_dict["sha256"] = utils.safe_value(env, "sha256")
-            sing_dict["post_sing_steps"] = utils.safe_value(env, "post-sing-steps")
+                docker_image = utils.safe_value(env, "image")
+                if not docker_image:
+                    errors.config_error("Definition for docker '{}' is missing the required 'image' property".format(docker_name))
 
-            registry_name = utils.safe_value(env, "registry")
-            # if not registry_name:
-            #     errors.config_error("Definition for docker '{}' is missing the required 'registry' property".format(docker_name))
+                sing_dict["sing_wrap"] = utils.safe_value(env, "sing-wrap")
+                sing_dict["sha256"] = utils.safe_value(env, "sha256")
+                sing_dict["post_sing_steps"] = utils.safe_value(env, "post-sing-steps")
+
+                registry_name = utils.safe_value(env, "registry")
+                # if not registry_name:
+                #     errors.config_error("Definition for docker '{}' is missing the required 'registry' property".format(docker_name))
 
             if registry_name:
                 docker_registry = self.get("external-services", registry_name, default_value=None)
