@@ -460,7 +460,7 @@ class HyperparameterExplorer(object):
 
         # Initialize the graphics.
         self.fig = plt.figure(figsize=(20,12))
-        self.fig.canvas.set_window_title('Hyperparameter Explorer')
+        self.fig.canvas.manager.set_window_title('Hyperparameter Explorer')
         self.left_pane_axes = self.fig.add_axes([0.0, 0.0, 1.0, 1.0])
         self.left_pane_axes.get_xaxis().set_visible(False)
         self.left_pane_axes.get_yaxis().set_visible(False)
@@ -663,7 +663,10 @@ class HyperparameterExplorer(object):
             if run.summary_val < self.hist_min_x:
                 self.hist_min_x = run.summary_val
         self.plot_num_points = max_reports
-        self.plot_x_inc = (self.plot_max_x - self.plot_min_x) / (self.plot_num_points - 1)
+        if self.plot_num_points > 1:
+            self.plot_x_inc = (self.plot_max_x - self.plot_min_x) / (self.plot_num_points - 1)
+        else:
+            self.plot_x_inc = 0.
 
     def populate_hparams(self):
         # Connect up the runs and hparams.
