@@ -10,9 +10,10 @@ from textwrap import wrap
 import time
 import datetime
 import tempfile
-from xtlib import node_helper, report_builder
 import numpy as np 
 
+import xtlib.xt_run as xt_run
+from xtlib import report_builder
 from xtlib.client import Client
 from xtlib.runner import Runner
 from xtlib.console import console
@@ -665,7 +666,7 @@ class ImplCompute(ImplBase):
     @flag("cancel", default=1, help="call the backend service to cancel the job")
     @flag("wrapup", default=1, help="wrapup the nodes and runs associated with the job")
     @example(task="cancel jobs job3 and job15", text="xt cancel job job3, job15")
-    @command(kwgroup="cancel", kwhelp="cancels active runs, specified by name, job, or target", help="cancels the specified jobs and their active or queued runs")
+    @command(kwgroup="cancel", kwhelp="cancels active runs, jobs, or requests", help="cancels the specified jobs and their active or queued runs")
     def cancel_job(self, job_names, workspace, max_workers, cancel, wrapup):
 
         for job_id in job_names:
@@ -697,6 +698,88 @@ class ImplCompute(ImplBase):
                 # cleanup the JOB
                 self.store.database.mark_job_cancelled(workspace, job_id)
 
+    #---- CANCEL REQUESTS command ----
+    @argument("names", type="str_list", help="the names of the requests to be cancelled")
+    @option("workspace", default="$general.workspace", help="the workspace that contains the requests")
+    @example(task="cancel requests request3 and request5", text="xt cancel requests request3, request5")
+    @command(kwgroup="cancel", kwhelp="cancels jobs, runs, requests", help="cancels job submission requests (created by xt run --request)")
+    def cancel_requests(self, names, workspace):
+
+        filter_dict = {"request_id": {"$in": names}}
+        records = self.store.database.get_info_for_requests(workspace, filter_dict)
+        for record in records:
+            status = record["status"]
+            name = record["request_id"]
+
+            if status == "requested":
+                username = self.config.get("general", "username")
+                self.store.update_request_status(record, "cancelled", username)
+                console.print("request cancelled: {}".format(name))
+
+            else:
+                console.print("request '{}' not pending; status: {}".format(name, status))
+
+    def send_email_to_requester(self, request_id, requester, approver, job_id):
+        team_dict = self.config.get("team")
+        requester_contacts = team_dict[requester]["contact"]
+        approver_contacts = team_dict[approver]["contact"]
+
+        # for now, assume all entries in approver_contacts are valid email names 
+        # TODO: add support for SMS text to phone numbers
+        subject = "approved: {} --> {}".format(request_id, job_id)
+
+        # make body HTML 
+        body = ""
+        body += "<b>job submission request:</b> {}".format(request_id)
+        body += "<br><b>approver by:</b> {}".format(approver)
+        body += "<br><b>created job_id:</b> {}".format(job_id)
+        sent = False
+
+        try:
+            utils.send_email(self.config, requester_contacts, approver_contacts, subject, body)
+            sent = True
+        except BaseException as ex:
+            console.print("email send failed: ex: {}".format(ex))
+
+    #---- APPROVE command ----
+    @argument("names", type="str_list", help="the names of the requests to be approved")
+    @option("workspace", default="$general.workspace", help="the workspace that contains the requests")
+    @example(task="approve requests request3 and request5", text="xt approve request3, request5")
+    @command(help="approves and submits a job for the specified requests ")
+    def approve(self, names, workspace):
+
+        filter_dict = {"request_id": {"$in": names}}
+        records = self.store.database.get_info_for_requests(workspace, filter_dict)
+        for record in records:
+            status = record["status"]
+            name = record["request_id"]
+
+            if status == "requested":
+                username = self.config.get("general", "username")
+                self.store.update_request_status(record, "approved", username)
+                console.print("request approved: {}".format(name))
+
+                xt_cmd = record["cmd"]
+                console.print("  running: " + xt_cmd)
+
+                # run the command, capturing the output so we can extract the job_id
+                console.set_capture(True, clear_captured=True)
+                xt_run.main(xt_cmd)
+
+                output_parts = console.set_capture(False)
+                job_id = output_parts[0].split(",", 1)[0]
+
+                # update record with job_id
+                record["job_id"] = job_id
+                self.store.database.update_requests_record(workspace, name, record)
+
+                requester = record["requested_by"]
+                self.send_email_to_requester(name, requester, username, job_id)
+                console.print("approval emailed to requester: {}".format(requester))
+
+            else:
+                console.print("request '{}' not pending; status: {}".format(name, status))
+
     # TODO: this will replace the 'view status' command
     #---- LIST QUEUES command ----
     @argument("target", help="the name of the compute target")
@@ -765,7 +848,7 @@ class ImplCompute(ImplBase):
     @hidden("username", default="$general.username", help="the username to log as the author of this run")
     @option("workspace", default="$general.workspace", help="the workspace to cancel runs in")
     @example(task="cancel all runs for current user on Philly", text="xt cancel all philly")
-    @command(kwgroup="cancel", kwhelp="cancels active runs, specified by name, job, or target", help="cancels all queued/active runs on the specified compute target")
+    @command(kwgroup="cancel", help="cancels all queued/active runs on the specified compute target")
     def cancel_all(self, target, username, workspace):
 
         backend = self.core.create_backend(target, username=username)
@@ -897,6 +980,7 @@ class ImplCompute(ImplBase):
     @option("aml_compute", type=str, default=DEFAULT_FROM_COMPUTE, help="specifies the name of the AML compute resource for the job")
     @hidden("capture_setup_cmds", type=bool, default="$logging.capture-setup-cmds", help="specifies if some setup cmds should output to a log file")
     @option("cluster", help="the name of the Philly cluster to be used")
+    @option("code", type=str, help="the source of a code snapshot (job id, file path, or blob path) to use in place of current code")
     @hidden("code-dirs", default="$code.code-dirs", help="paths to the main code directory and dependent directories")
     @hidden("code-omit", default="$code.code-omit", help="the list wildcard patterns to omit uploading from the code files")
     @flag("code-upload", default="$code.code-upload", help="when true, code is uploaded to job and download for each run of job")
@@ -911,6 +995,7 @@ class ImplCompute(ImplBase):
     @hidden("delay-evaluation", default="$early-stopping.delay-evaluation", type=int, help="number of evals (metric loggings) to wait before applying early stopping policy")
     @option("description", help="your description of this run")
     @flag("direct-run", default="$general.direct-run", help="when True, the script is run without the controller (on Philly or AML)")
+    @option("display-name", default="$general.display-name", type=str, help="the singularity display name for this job")
     @flag("distributed", default="$general.distributed", help="when True, the multiple nodes will be put into distributed training mode")
     @hidden("distributed-training", default="$aml-options.distributed-training", help="the name of the backend process to use for distributed training")
     @option("docker", type=str, help="the name of the docker entry to use with the target (from one of the entries in the config file [dockers] section")
@@ -927,7 +1012,6 @@ class ImplCompute(ImplBase):
     @hidden("fn_generated_config", default="$hyperparameter-search.fn-generated-config", type=str, help="the name for the generated HP config file")
     @hidden("framework", default="$aml-options.framework", help="the name of the framework to be installed for the run (pytorch, tensorflow, chainer)")
     @hidden("fw-version", default="$aml-options.fw-version", help="the framework version number")
-    @option("display-name", default="$general.display-name", type=str, help="the singularity display name for this job")
     @option("goal-metric", default="$hyperparameter-search.goal-metric", type=float, help="goal for primary metrics (stops CCD hp search when reached)")
     #@hidden("grok-server", default="$logging.grok-server", help="the ip address of the grok-server to be used for mirroring")
     @flag("hold", help="when True, the Azure Pool (VM's) are held open for debugging)")
@@ -970,6 +1054,7 @@ class ImplCompute(ImplBase):
     @option("queue", type=str, help="the name of the Philly queue to use when submitting this job")
     @hidden("remote-control", default="$general.remote-control", help="specifies if XT controller will listen for XT client commands")
     @hidden("report-rollup", default="$run-reports.report-rollup", help="whether to rollup metrics by primary metric or just use last reported metric set")
+    @flag("request", help="instead of submitting this job directly, create a request to have it submitted by a team member with approval authority")
     @option("resume-name", help="when resuming a run, this names the previous run")
     @option("runs", default=None, type=int, help="the total number of runs across all nodes (for hyperparameter searches)")
     @option("runs-per-set", default="$hyperparameter-search.runs-per-set", type=int, help="how many runs to do for each CCD hp set explored")
@@ -1043,20 +1128,21 @@ class ImplCompute(ImplBase):
 
         with tempfile.TemporaryDirectory(prefix="import-") as temp_dir:
             runner = Runner(self.config, self.core, temp_dir)
+            result = runner.process_run_command(args)
 
-            cmds, run_specs, using_hp, using_aml_hparam, sweeps_text, pool_info, job_id = \
-                runner.process_run_command(args)
+        if result:
+            cmds, run_specs, using_hp, using_aml_hparam, sweeps_text, pool_info, job_id = result
 
-        monitor = args["monitor"]
-        escape = args["escape"]
-        workspace = args["workspace"]
+            monitor = args["monitor"]
+            escape = args["escape"]
+            workspace = args["workspace"]
 
-        if monitor == "same":
-            self.monitor(name=job_id, workspace=workspace, escape=escape)
+            if monitor == "same":
+                self.monitor(name=job_id, workspace=workspace, escape=escape)
 
-        elif monitor in ["bg", "new"]:
-            store_name = self.config.data["store"]
-            cmd = "xt --echo --store={} monitor {} --workspace={}".format(store_name, job_id, workspace)
-            noactivate = (monitor=="bg")
-            process_utils.run_cmd_in_new_console(cmd, noactivate=noactivate)
+            elif monitor in ["bg", "new"]:
+                store_name = self.config.data["store"]
+                cmd = "xt --echo --store={} monitor {} --workspace={}".format(store_name, job_id, workspace)
+                noactivate = (monitor=="bg")
+                process_utils.run_cmd_in_new_console(cmd, noactivate=noactivate)
 
