@@ -350,8 +350,8 @@ class MyHandler(FileSystemEventHandler):
             fn_rel = fn[plen:]
             fn_rel = fn_rel.replace("\\", "/")
 
-            if self.show_calls:
-                console.print("on_any_event BEFORE: fn: {}, fn_rel: {}".format(fn, fn_rel))
+            # if self.show_calls:
+            #     console.print("on_any_event BEFORE: fn: {}, fn_rel: {}".format(fn, fn_rel))
 
             if "/" in fn_rel:
                 # it is a multi-dir path; use all but last folder of fn_rel
@@ -407,9 +407,9 @@ class MyHandler(FileSystemEventHandler):
                         src_blob_path = self.build_blob_path(fn_src, event_type, is_dir)
                         dest_blob_path = self.build_blob_path(fn_dest, event_type, is_dir)
 
-                        if self.show_calls:
-                            console.print("on_any_event AFTER: fn_src: {}, fn_dest: {}, src_blob_path: {}, dest_blob_path: {}, event_type: {}, is_dir: {}".format( \
-                                fn_src, fn_dest, src_blob_path, dest_blob_path, event_type, is_dir))
+                        # if self.show_calls:
+                        #     console.print("on_any_event AFTER: fn_src: {}, fn_dest: {}, src_blob_path: {}, dest_blob_path: {}, event_type: {}, is_dir: {}".format( \
+                        #         fn_src, fn_dest, src_blob_path, dest_blob_path, event_type, is_dir))
 
                         if self.delay_queue:
                             # BUFFERING CHANGES; send the change to the bg worker via our queue
