@@ -52,93 +52,96 @@ node_stats_props = \
         'restarts': 1, 'db_retries': 1, 'storage_retries': 1
     }
 
+all_node_props = {**node_info_props, **node_stats_props}
+
+user_to_actual = { 
+    # NODE_INFO: creation time properties
+    "ws_name": "ws_name",
+    "job_id": "job_id", 
+    "node_name": "node_name",
+    "node_id": "node_id",
+    "node_index": "node_index",
+    "node_num": "node_num",
+    "exper_name": "exper_name",
+    "target": "target",
+    "aml_compute": "aml_compute",
+    "compute_target": "compute_target",
+    "total_runs": "total_runs",
+    "box_name": "box_name",
+    "run_name": "run_name",
+    "secret": "secret",
+    "service_info": "service_info",
+
+    # NODE_STATS: updatable properties
+    "status": "node_status",
+    "tags": "tags",
+    "controller_port": "controller_port", 
+    "ip_addr": "ip_addr",
+    "completed_runs": "completed_runs",
+    "error_runs": "error_runs",
+    "running_runs": "running_runs",
+
+    "create_time": "create_time",
+    "prep_start_time": "prep_start_time",
+    "app_start_time": "app_start_time",
+    "post_start_time": "post_start_time",
+    "post_end_time": "post_end_time",
+
+    "queue_duration": "queue_duration",
+    "prep_duration": "prep_duration",
+    "app_duration": "app_duration",
+    "post_duration": "post_duration",
+
+    "restarts": "restarts",
+    "db_retries": "db_retries",
+    "storage_retries": "storage_retries",
+}
+
+std_cols_desc = {
+    # the key here is the USER friendly name (not the physical name)
+    "ws_name": "the workspace that the associated job is defined within",
+    "job": "the id of the job that this node belongs to",
+    "name": "the name of the node (jobNN/N)",
+    "node_id": "the id of the node, within the job",
+    "node_index": "the index of the node, within the job",
+    "node_num": "a combination of the job/node that is used to sort by the node name",
+    "exper_name": "experiment name associated with this node's job",
+    "target": "the name of the AML compute for this node (legacy column)",
+    "aml_compute": "the name of the AML compute for this node",
+    "compute_target": "the name of the compute target for this node",
+    "total_runs": "the number of runs assigned to this node",
+    "box_name": "the user-friendly name of this node",
+    "run_name": "the name of the flat or parent run associated with this node",
+    "secret": "the key for connecting with this node",
+    "service_info": "a JSON blob of information used by the backend service for this node",
+
+    # dynamic properties (get updated at various stages of the node)
+    "status": "one of: created, running, completed",
+    "tags": "the user-defined tags that have been added to the node record",
+    "controller_port": "the controller port used by XT to talk with the node", 
+    "ip_addr": "the IP address used by XT to talk with the node",
+    "completed_runs": "the number of runs that have completed (with or without errors)",
+    "error_runs": "the number of runs that have terminated with an error",
+    "running_runs": "the number of this job's runs that are current running",
+
+    "create_time": "when the node and job were created",
+    "prep_start_time": "when the XT setup script started",
+    "app_start_time": "when the XT controller began running on the node",
+    "post_start_time": "when the POST section of the XT setup script started",
+    "post_end_time": "when the XT setup script completed",
+
+    "queue_duration": "the number of seconds that this node was queued before running",
+    "prep_duration": "the number of seconds that XT spent setting up the node",
+    "app_duration": "the number of seconds the XT controller (and user scripts) spent running",
+    "post_duration": "the number of seconds the POST section of the XT setup script spend running",
+
+    "restarts": "the number of times this node has been restarted",
+    "db_retries": "the total number of database retries performed by the runs of this node",
+    "storage_retries": "the total number of storage retries performed by the runs of this node",
+}
+
 def get_node_property_dicts():
     # user-friendly property names for nodes
-    user_to_actual = { 
-        # NODE_INFO: creation time properties
-        "ws_name": "ws_name",
-        "job_id": "job_id", 
-        "node_name": "node_name",
-        "node_id": "node_id",
-        "node_index": "node_index",
-        "node_num": "node_num",
-        "exper_name": "exper_name",
-        "target": "target",
-        "aml_compute": "aml_compute",
-        "compute_target": "compute_target",
-        "total_runs": "total_runs",
-        "box_name": "box_name",
-        "run_name": "run_name",
-        "secret": "secret",
-        "service_info": "service_info",
-
-        # NODE_STATS: updatable properties
-        "status": "node_status",
-        "tags": "tags",
-        "controller_port": "controller_port", 
-        "ip_addr": "ip_addr",
-        "completed_runs": "completed_runs",
-        "error_runs": "error_runs",
-        "running_runs": "running_runs",
-
-        "create_time": "create_time",
-        "prep_start_time": "prep_start_time",
-        "app_start_time": "app_start_time",
-        "post_start_time": "post_start_time",
-        "post_end_time": "post_end_time",
-
-        "queue_duration": "queue_duration",
-        "prep_duration": "prep_duration",
-        "app_duration": "app_duration",
-        "post_duration": "post_duration",
-
-        "restarts": "restarts",
-        "db_retries": "db_retries",
-        "storage_retries": "storage_retries",
-    }
-
-    std_cols_desc = {
-        # the key here is the USER friendly name (not the physical name)
-        "ws_name": "the workspace that the associated job is defined within",
-        "job": "the id of the job that this node belongs to",
-        "name": "the name of the node (jobNN/N)",
-        "node_id": "the id of the node, within the job",
-        "node_index": "the index of the node, within the job",
-        "node_num": "a combination of the job/node that is used to sort by the node name",
-        "exper_name": "experiment name associated with this node's job",
-        "target": "the name of the AML compute for this node (legacy column)",
-        "aml_compute": "the name of the AML compute for this node",
-        "compute_target": "the name of the compute target for this node",
-        "total_runs": "the number of runs assigned to this node",
-        "box_name": "the user-friendly name of this node",
-        "run_name": "the name of the flat or parent run associated with this node",
-        "secret": "the key for connecting with this node",
-        "service_info": "a JSON blob of information used by the backend service for this node",
-
-        # dynamic properties (get updated at various stages of the node)
-        "status": "one of: created, running, completed",
-        "tags": "the user-defined tags that have been added to the node record",
-        "controller_port": "the controller port used by XT to talk with the node", 
-        "ip_addr": "the IP address used by XT to talk with the node",
-        "completed_runs": "the number of runs that have completed (with or without errors)",
-        "error_runs": "the number of runs that have terminated with an error",
-        "running_runs": "the number of this job's runs that are current running",
-
-        "create_time": "when the node and job were created",
-        "prep_start_time": "when the XT setup script started",
-        "app_start_time": "when the XT controller began running on the node",
-        "post_start_time": "when the POST section of the XT setup script started",
-        "post_end_time": "when the XT setup script completed",
-
-        "queue_duration": "the number of seconds that this node was queued before running",
-        "prep_duration": "the number of seconds that XT spent setting up the node",
-        "app_duration": "the number of seconds the XT controller (and user scripts) spent running",
-        "post_duration": "the number of seconds the POST section of the XT setup script spend running",
-
-        "restarts": "the number of times this node has been restarted",
-        "db_retries": "the total number of database retries performed by the runs of this node",
-        "storage_retries": "the total number of storage retries performed by the runs of this node",
-    }
 
     return user_to_actual, std_cols_desc
 
