@@ -12,6 +12,19 @@
 
     - XT docs should include psutil as a prerequisite for each pool box
 '''
+
+'''
+PSM order of operations:
+    - setup directories
+
+    - enter scheduling loop:
+        - every 1 second:
+            - get list of files in queue
+            - find first .zip file in queue - if found:
+                - if # active jobs < max:
+                    - start job from .zip file
+
+'''
 import os
 import time
 import shutil
@@ -99,20 +112,45 @@ def get_controller_wrapped_counts():
         
     return controller_count, wrapped_count, docker_count
 
-def start_async_run_detached(cmd, working_dir, fn_stdout):
-    DETACHED_PROCESS = 0x00000008    # if visible else 0
-    CREATE_NO_WINDOW = 0x08000000
+def start_async_run_detached(cmd_parts, working_dir):
+    cmd = " ".join(cmd_parts)
+
+    merge_batch_logs = True
+    logs_dir = "{}/logs".format(working_dir)
+    os.makedirs(logs_dir, exist_ok=True)
+
+    if merge_batch_logs:
+        shell_cmd = "{} > ./logs/stdboth.txt 2>&1".format(cmd)
+    else:
+        shell_cmd = "{} > ./logs/stdout.txt 2> ./logs/stderr.txt".format(cmd)
+
+    # write shell_cmd to start_run.sh (since we want to use shell-level redirection)
+    fn = "{}/start_run.sh".format(working_dir)
+    with open(fn, "wt") as outfile:
+        outfile.write(shell_cmd)
+
+    final_cmd_parts = ["/bin/bash", fn]
+
+    log_print("  starting ENTRY:")
+    log_print("    shell_cmd={}".format(shell_cmd))
+    log_print("    cwd={}".format(working_dir))
+    log_print("    final_cmd_parts={}".format(final_cmd_parts))
+    log_print()
+
+    # run async
+    if is_windows:
+        DETACHED_PROCESS = 0x00000008    # if visible else 0
+        CREATE_NO_WINDOW = 0x08000000
     
-    with open(fn_stdout, 'w') as output:
+        cflags = CREATE_NO_WINDOW  # | DETACHED_PROCESS
+        p = subprocess.Popen(final_cmd_parts, cwd=working_dir, creationflags=cflags)
 
-        if is_windows:
-            cflags = CREATE_NO_WINDOW  # | DETACHED_PROCESS
-            p = subprocess.Popen(cmd, cwd=working_dir, stdout=output, stderr=subprocess.STDOUT, creationflags=cflags)
+    else:
+        # linux
+        p = subprocess.Popen(final_cmd_parts, cwd=working_dir)
+
+    return p    
 
-        else:
-            # linux
-            p = subprocess.Popen(cmd, cwd=working_dir, stdout=output, stderr=subprocess.STDOUT)
-    return p
 
 def start_entry(fn_entry):
     '''
@@ -171,22 +209,19 @@ def start_entry(fn_entry):
         run_name = parts[3]
         node_index = parts[4]
 
-        fn_base_entry = os.path.splitext(fn_entry)[0]
-        fn_log = os.path.join(PSM_LOGDIR, fn_base_entry + ".log")
+        #fn_base_entry = os.path.splitext(fn_entry)[0]
+        #fn_log = os.path.join(PSM_LOGDIR, fn_base_entry + ".log")
 
         # force it to be a legal filename for current platform (correct slashes)
-        fn_log = os.path.abspath(fn_log)
+        #fn_log = os.path.abspath(fn_log)
 
         if fn_wrapper.endswith(".bat"):
-            cmd_parts = [fn_wrapper, node_index, run_name, fn_log]
+            cmd_parts = [fn_wrapper, node_index, run_name]
         else:
-            cmd_parts = ["bash", "--login", fn_wrapper, node_index, run_name, fn_log]
-
-        log_print("  starting ENTRY, cmd_parts={}".format(cmd_parts))
-        log_print()
+            cmd_parts = ["bash", "--login", fn_wrapper, node_index, run_name]
 
         # run WRAPPED script on local box
-        start_async_run_detached(cmd_parts, CWD, fn_log)
+        start_async_run_detached(cmd_parts, CWD)
 
     except BaseException as ex:
         # log and move on to next entry
@@ -246,6 +281,5 @@ def main():
                 # start processing oldest entry
                 start_entry(fn_entry)
 
-
 if __name__ == "__main__":
     main()
