@@ -24,9 +24,13 @@ from xtlib import constants
 from xtlib.hp_set import HpSetFormatter
 from xtlib.helpers import html_writer
 
-def format_value(fmt, value):
+def safe_format(fmt, value, col_width):
     if value is None:
         text = ""
+    elif value == "":
+        text = col_width*" "
+    elif fmt is None:
+        text = str(value)
     else:
         text = fmt.format(value)
 
@@ -328,6 +332,25 @@ class ReportBuilder():
 
         return first, last
 
+    def get_user_col_info(self, user_cols, col_name):
+
+        for col in user_cols:
+
+            name = col
+            user_name = None
+            fmt = None
+
+            if "=" in name:
+                name, user_name = col.split("=", 1)
+
+            if ":" in name:
+                name, fmt = name.split(":", 1)
+
+            if name == col_name:
+                return name, fmt, user_name
+            
+        return None, None, None
+
     def get_db_records(self, database, filter_dict, workspace, which, actual_to_user,
             col_dict=None, col_names_are_external=True, flatten_records=True, need_alive=False, 
             user_columns=None, hide_empty_cols=None, args=None):
@@ -355,21 +378,33 @@ class ReportBuilder():
         # use database to do all of the work (query, sort, first/last)
         sort_col = utils.safe_value(args, "sort")   
         group_col = utils.safe_value(args, "group")   
+        mean_cols = utils.safe_value(args, "mean")   
+        add_cols = utils.safe_value(args, "add_columns")   
+
         sort_dir = 1
+        hp_set_display = utils.safe_value(args, "hp_set_display")
+
+        # if mean_col and not group_col:
+        #     errors.syntax_error("cannot use --mean without setting the grouping column (--group)")
 
         user_to_actual = {value: key for key, value in actual_to_user.items()}
 
+        if hp_set_display and hp_set_display != "hidden":
+            # add to user_columns, if not already there
+            name, _, _ = self.get_user_col_info(user_columns, "hp_set")
+            if not name:
+                user_columns.append("hp_set")
+
+        if group_col == "hp_set" and hp_set_display == "hidden":
+            # nice default for grouping
+            hp_set_display = "changed"
+            args["hp_set_display"] = hp_set_display
+
         # group vs. sort
         if group_col:
-            def _get_name(text): 
-                if "=" in text:
-                    text = text.split("=", 1)[0]
-                if ":" in text:
-                    text = text.split(":", 1)[0]
-                return text
-
-            user_col_names = [_get_name(col_format) for col_format in user_columns]
-            if not group_col in user_col_names:
+            # add to user_columns, if not already there
+            name, _, _ = self.get_user_col_info(user_columns, group_col)
+            if not name:
                 user_columns.append(group_col)
 
             # if sort_col and group_col != sort_col:
@@ -399,9 +434,9 @@ class ReportBuilder():
             elif not "." in sort_col:
                 # translate name of std col from user-friendly version to logged version
 
-                if not sort_col in user_to_actual:
-                    errors.general_error("unknown standard property: {} (did you mean metrics.{}, hparams.{}, or tags.{}?)". \
-                        format(sort_col, sort_col, sort_col, sort_col))
+                # if not sort_col in user_to_actual:
+                #     errors.general_error("unknown standard property: {} (did you mean metrics.{}, hparams.{}, or tags.{}?)". \
+                #         format(sort_col, sort_col, sort_col, sort_col))
 
                 sort_col = user_to_actual[sort_col]
 
@@ -413,8 +448,21 @@ class ReportBuilder():
                 first = last
 
             # ensure we only ask for records where sort_col exists, or else we MIGHT end up with less than LIMIT records
-            if sort_col != "run_num" and not sort_col in filter_dict:
-                filter_dict[sort_col] = { "$exists": True}
+            # don't think we need this anymore (rfernand, Nov-11-2022)
+            # if sort_col != "run_num" and not sort_col in filter_dict:
+            #     filter_dict[sort_col] = { "$exists": True}
+
+        # validate specified col names
+        self.validate_col(which, "--sort", sort_col)
+        self.validate_col(which, "--group", group_col)
+
+        if mean_cols:
+            for mc in mean_cols:
+                self.validate_col(which, "--mean", mc)
+
+        if add_cols:
+            for ac in add_cols:
+                self.validate_col(which, "--add-columns", ac)
 
         # always include ws_name (for now)
         if workspace and not "ws_name" in filter_dict:
@@ -445,12 +493,18 @@ class ReportBuilder():
         if which == "runs":
             records = database.get_filtered_sorted_run_info(workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
                 count_runs, buffer_size, hide_empty_cols=hide_empty_cols)
+
         elif which == "jobs":
             records = database.get_filtered_sorted_job_info(workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
                 count_runs, buffer_size, hide_empty_cols=hide_empty_cols)
+
         elif which == "nodes":
             records = database.get_filtered_sorted_node_info(workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
                 count_runs, buffer_size, hide_empty_cols=hide_empty_cols)
+
+        elif which == "requests":
+            records = database.get_filtered_sorted_request_info(workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
+                count_runs, buffer_size, hide_empty_cols=hide_empty_cols)
         else:
             errors.internal_error("unrecognized value for which: {}".format(which))
 
@@ -463,6 +517,12 @@ class ReportBuilder():
             # pull out standard cols, translating from actual to user-friendly names
             records = [self.translate_record(rec, actual_to_user) for rec in records if rec]
 
+        exclude_from_hp_set = utils.safe_value(args, "exclude_from_hp_set") 
+
+        # fixup null hp_set values using all logged hparams (don't filter by user_columns)
+        if group_col == "hp_set" or (hp_set_display and hp_set_display != "hidden"):
+            self.fixup_legacy_hp_set_nulls(records, exclude_from_hp_set)
+
         if flatten_records:
             # pull out requested cols, flattening nested values to their dotted names
             records = self.flatten_records(records, sort_col, need_alive, which, user_columns=user_columns, args=args)
@@ -477,13 +537,51 @@ class ReportBuilder():
 
         return records, limiter, limiter_value
 
+    def validate_col(self, which, name, col_name):
+        '''
+        col_name could be in either user or actual names,
+        depending on the caller.  we check for both.
+        '''
+        # prevent circular imports with JIT usage
+        from xtlib import run_helper
+        from xtlib import job_helper
+        from xtlib import node_helper
+        from xtlib import request_helper
+
+        found = False
+
+        if not col_name:
+            found = True
+        elif col_name.startswith("hparams."):
+            found = True
+        elif col_name.startswith("metrics."):
+            found = True
+        elif col_name.startswith("tags."):
+            found = True
+        elif which == "runs":
+            found = col_name in run_helper.user_to_actual or col_name in run_helper.all_run_props
+        elif which == "jobs":
+            found = col_name in job_helper.user_to_actual or col_name in job_helper.all_job_props
+        elif which == "requests":
+            found = col_name in request_helper.user_to_actual or col_name in request_helper.all_job_props
+        elif which == "nodes":
+            found = col_name in node_helper.user_to_actual or col_name in node_helper.all_node_props
+        else:
+            errors.internal_error("validate_col(): 'which' value not yet supported: {}".format(which))
+
+        if not found:
+            errors.syntax_error("unknown standard column specified for {}: {} (did you mean metrics.{}, hparams.{}, or tags.{}?)". \
+                format(name, col_name, col_name, col_name, col_name))
+
     def get_user_columns(self, requested_list, args):
         if requested_list is None:
             requested_list = args["columns"]
             
         add_cols = utils.safe_value(args, "add_columns")
         if add_cols:
-            requested_list += add_cols
+            for ac in add_cols:
+                if not ac in requested_list:
+                    requested_list.append(ac)
 
         return requested_list
 
@@ -526,7 +624,7 @@ class ReportBuilder():
         was_exported = False
         max_column_width = utils.safe_value(args, "max_width")
 
-        hp_set_display = utils.safe_value(args, "hp_set")
+        hp_set_display = utils.safe_value(args, "hp_set_display")
         mean_cols = utils.safe_value(args, "mean")
         group_by = utils.safe_value(args, "group")
         sort_by = utils.safe_value(args, "sort")
@@ -539,83 +637,18 @@ class ReportBuilder():
             first = None
             last = None
 
+        # fixup hp_set cols in records, as per hp_set_display
+        self.format_hp_set_values(records, user_columns, hp_set_display)
+
+        # build column values for: QUEUED, DURATION
+        records = self.add_dynamic_cols(records, user_columns)
+
         if mean_cols:
             # produce new mean-calculated columns (new records)
-            records = self.add_dynamic_cols(records, user_columns)
             records = self.create_mean_records(user_columns, records, mean_cols, group_by, sort_by, reverse, first, last)
             if args["flat"]:
                 group_by = None
 
-        # from the current set of records, calculate the set of hyperparameters that change between runs
-        self.hp_set_formatter.build_hp_set_names(records)
-        hp_set_index = user_columns.index("hp_set") if "hp_set" in user_columns else -1
-
-        # if hp_set_display != "simple_plus_columns":
-        #     # remove all hp_set column/format info
-        #     cols = list(user_columns)
-        #     for col in cols:
-        #         if col == "hp_set" or col.startswith("hp_set=") or col.startswith("hp_set:"):
-        #             user_columns.remove(col)
-
-        def _set_or_append(values, value, index): 
-            if index > -1:
-                values[index] = value
-            else:
-                values.append(value)
-
-        '''
-        The --hp_set_display controls how a hp-set is incorporated into your list runs report.  A hp-set
-        is a collection of all the hyperparameters in a run.  You can include only the hyperparameters whose
-        values changed in 1 or more runs, or all of the hyperparameters.
-
-            changed:                adds a JSON column for the changed hparams
-            full:                   adds a JSON column for all of the hparams
-            simple:                 adds a column for the name of the hp_set (e.g., hp_set_1)
-            columns:                adds a column for each changed hparam
-            user-columns:           adds a column for each hparam specified by the user (run-reports, named-columns, etc.)
-            simple_plus_columns:    adds a column for each changed hparam, and a colum for the hp-set name
-        '''
-        if hp_set_display == "changed":
-            _set_or_append(user_columns, "hp_set:$hpset_changed", hp_set_index)
-
-        if hp_set_display == "simple" or hp_set_display == "simple_plus_columns":
-            # "simpl"
-            _set_or_append(user_columns, "hp_set:$hpset_simple", hp_set_index)
-
-        if hp_set_display == "full":
-            _set_or_append(user_columns, "hp_set", hp_set_index)
-
-        if hp_set_display == "columns" or hp_set_display == "simple_plus_columns":
-            if self.hp_set_formatter.hp_common_set:
-                # for each run record, convert hp_set column into new columns in the record 
-                for record in records:
-                    if "hp_set" in record:
-                        hp_set_str = record["hp_set"]
-                        hp_set = self.hp_set_formatter.format_hpset_changed(hp_set_str)
-
-                        for hp, value in hp_set.items():
-                            full_hp_name = "hparams." + hp
-                            record[full_hp_name] = value
-
-                        if hp_set_display == "columns":
-                            del record["hp_set"]
-
-        if hp_set_display == "user-columns":
-            if self.hp_set_formatter.hp_common_set:
-                # for each run record, convert hp_set column into new columns in the record 
-                for record in records:
-                    if "hp_set" in record:
-                        hp_set_str = record["hp_set"]
-                        hp_set = self.hp_set_formatter.parse_and_sort_hp_set(hp_set_str)
-
-                        for hp, value in hp_set.items():
-                            full_hp_name = "hparams." + hp
-                            if full_hp_name in user_columns:
-                                record[full_hp_name] = value
-
-                        del record["hp_set"]
-
-
         # NOTE: user_col_args is a list here, so that we can support multiple instances of same col in long report lines
         avail_list, actual_cols, user_col_args  = self.get_avail_actual_user_cols(records, user_columns, args)
 
@@ -641,14 +674,100 @@ class ReportBuilder():
         else:
             number_groups =  utils.safe_value(args, "number_groups")
 
+            group_by_fmt = None
+            if group_by:
+                _, group_by_fmt, _ = self.get_user_col_info(user_columns, group_by)
+
+                if group_by_fmt:
+                    group_by_fmt = "{:" + group_by_fmt + "}" 
+
             text, row_count = self.build_formatted_table(records, avail_cols=avail_list, col_list=actual_cols, 
                 report_type=report_type, group_by=group_by, number_groups=number_groups, 
-                max_col_width=max_column_width)
+                max_col_width=max_column_width, group_by_fmt=group_by_fmt)
 
             lines = text.split("\n")
 
         return lines, row_count, was_exported
 
+    def format_hp_set_values(self, records, user_columns, hp_set_display):
+        '''
+        The hp_set column is part of the run_info table record created for each run.  hp_set is a string     
+        version of a dictionary containing all of the hyperparameters (and their associated values), that were
+        a result of the hyperparameter search, the cmdline arguments for your script, or the logged hyperparameters 
+        for your run.
+
+        The --hp-set-display option controls how hp_set is used and formatted in the list runs report:
+        
+            changed:                hp_set is transformed to only include hp names/values unique to this run
+            full:                   hp_set is unchanged
+            simple:                 hp_set is changed to a simple name for its unique set of values (e.g., hp_set_1)
+            columns:                adds a new column to the report for each changed hparam
+            user-columns:           adds a new column for changed hparam if column has been specified by user (run-reports, named-columns, etc.)
+            simple_plus_columns:    adds a column for each changed hparam, and a column for the hp_set name
+        '''
+
+        # from the current set of records, calculate the set of hyperparameters that change between runs
+        self.hp_set_formatter.build_hp_set_names(records)
+        hp_set_index = user_columns.index("hp_set") if "hp_set" in user_columns else -1
+
+        def _set_or_append(values, value, index): 
+            if index > -1:
+                values[index] = value
+            else:
+                values.append(value)
+
+        if hp_set_display == "changed":
+            _set_or_append(user_columns, "hp_set:$hpset_changed", hp_set_index)
+
+        if hp_set_display == "simple" or hp_set_display == "simple_plus_columns":
+            # "simpl"
+            _set_or_append(user_columns, "hp_set:$hpset_simple", hp_set_index)
+
+        if hp_set_display == "full":
+            _set_or_append(user_columns, "hp_set", hp_set_index)
+
+        # add new HP columns to each record
+        if hp_set_display == "columns" or hp_set_display == "simple_plus_columns":
+            if self.hp_set_formatter.hp_sets_processed:
+
+                # add each hparam col that has changed to user_columns
+                for col in self.hp_set_formatter.hp_changed_keys:
+                    self.add_to_user_columns_if_needed(user_columns, "hparams." + col)
+
+                for record in records:
+                    # for each changed hparam, add to run record
+                    if "hp_set" in record:
+                        hp_set_str = record["hp_set"]
+                        hp_set = self.hp_set_formatter.format_hpset_changed(hp_set_str)
+
+                        for hp, value in hp_set.items():
+                            full_hp_name = "hparams." + hp
+                            record[full_hp_name] = value
+
+                        if hp_set_display == "columns":
+                            del record["hp_set"]
+
+        # add new HP columns to each record
+        if hp_set_display == "user-columns":
+            if self.hp_set_formatter.hp_sets_processed:
+                # for each run record, add new columns for each changed hparam that is found in user_columns
+                for record in records:
+                    if "hp_set" in record:
+                        hp_set_str = record["hp_set"]
+                        hp_set = self.hp_set_formatter.parse_and_sort_hp_set(hp_set_str)
+
+                        for hp, value in hp_set.items():
+                            full_hp_name = "hparams." + hp
+                            if full_hp_name in user_columns:
+                                record[full_hp_name] = value
+
+                        del record["hp_set"]
+
+    def add_to_user_columns_if_needed(self, user_columns, col_name):
+        name, _, _ = self.get_user_col_info(user_columns, col_name)
+        if not name:
+            user_columns.append(col_name)
+
     def export_records(self, fn_report, records, col_list, sep_char):
 
         lines = []
@@ -805,7 +924,7 @@ class ReportBuilder():
 
     def build_formatted_table(self, records, avail_cols=None, col_list=None, max_col_width=None, 
         report_type="run-reports", group_by=None, number_groups=False, indent=None, print_report=False, 
-        copy_as_html=False, skip_lines=None):
+        copy_as_html=False, skip_lines=None, group_by_fmt=None):
 
         '''
         Args:
@@ -818,10 +937,6 @@ class ReportBuilder():
             Builds a nicely formatted text table from a set of records.
         '''
 
-        if group_by and group_by in col_list:
-            # remove group_by columns from those display in group records
-            col_list.remove(group_by)
-
         time_col_names = ["created", "started", "ended", "time"]
         duration_col_names = ["duration", "queued", "queue_duration", "prep_duration",
             "app_duration", "post_duration"]
@@ -856,6 +971,10 @@ class ReportBuilder():
         if not col_list:
             col_list = avail_cols
 
+        if group_by and group_by in col_list:
+            # remove group_by columns from those display in group records
+            col_list.remove(group_by)
+
         col_space = 2               # spacing between columns
         col_infos = []              # {width: xxx, value_type: int/float/str, is_numeric: true/false}
         header_line = None
@@ -874,13 +993,14 @@ class ReportBuilder():
             max_precision = precision 
 
             # examine all records for determining max col_widths
+            user_col = col
+            user_fmt = None
+
             if self.user_col_args:
                 user_args = self.get_user_col_arg(col)
-                user_col = user_args["user_name"]
-                user_fmt = user_args["user_fmt"] 
-            else:
-                user_col = col
-                user_fmt = None
+                if user_args:
+                    user_col = user_args["user_name"]
+                    user_fmt = user_args["user_fmt"] 
 
             float_fmt = "{:." + str(precision) + "f}"
 
@@ -906,9 +1026,12 @@ class ReportBuilder():
                         value = self.format_duration(value, col, record)
 
                     elif col in time_col_names:
-                        if isinstance(value, str):
-                            value = arrow.get(value)
-                        value = value.format('YYYY-MM-DD @HH:mm:ss')
+                        if value is None:
+                            value = ""
+                        else:
+                            if isinstance(value, str):
+                                value = arrow.get(value)
+                            value = value.format('YYYY-MM-DD @HH:mm:ss')
 
                     # for sql, convert float types that are really integer values to int types
                     if isinstance(value, float) and value == int(value) and user_fmt is None and value_type is str:
@@ -922,10 +1045,7 @@ class ReportBuilder():
 
                     if user_fmt:
                         # user provided explict format for this column
-                        if "$" in user_fmt:
-                            value_str, align = self.xt_custom_format(user_fmt, value)
-                        else:
-                            value_str = format_value(user_fmt, value)
+                        value_str, align = self.format_value(user_fmt, value, col_width)
 
                     elif isinstance(value, float):
                         if col == "metrics.best-eval-acc":
@@ -997,13 +1117,24 @@ class ReportBuilder():
             row_count = 0
             group_count = 0
 
-            grouped_records = self.group_by(records, group_by)
+            grouped_records = self.group_by(records, group_by, group_by_fmt)
             for i, (group, grecords) in enumerate(grouped_records.items()):
 
+                by = self.remove_col_prefix(group_by)
+                fmt_group, need_newline = self.format_group_value(group)
+
+                if need_newline:
+                    text += "\n"
+
                 if number_groups:
-                    text += "\n{}. {}:\n".format(i+1, group)
+                    #text += "\n{}. {}:\n".format(i+1, group)
+                    text += "\ngroup #{}: [{}: {}]\n".format(i+1, by, fmt_group)
                 else:
-                    text += "\n{} {}:\n".format(group_by, group)
+                    #text += "\n{} {}:\n".format(group_by, group)
+                    text += "\ngroup: [{}: {}]\n".format(by, fmt_group)
+
+                if need_newline:
+                    text += "\n"
 
                 txt, rc = self.generate_report(col_infos, grecords, right_align_num_cols, uppercase_hdr_cols, truncate_with_ellipses, 
                     col_space, duration_col_names, time_col_names, significance, max_fixed_length, precision, skip_lines=skip_lines)
@@ -1036,7 +1167,39 @@ class ReportBuilder():
 
         return text, row_count
 
-    def create_mean_records_core(self, records, mean_cols, group_by):
+    def format_group_value(self, value):
+
+        def simple_fmt(v):
+            if isinstance(v, str):
+                v = utils.make_numeric_if_possible(v)
+            if isinstance(v, float) and v == int(v):
+                v = str(int(v))
+            else:
+                v = str(v)
+            return v
+
+        need_newline = False
+
+        if value.startswith("{"):
+            # try an easier to read format of group dict 
+            newlines = False
+
+            import ast 
+            value = ast.literal_eval(value)
+            
+            max_name_len = max([len(key) for key in value])
+
+            if newlines:
+                value = "".join(["\n    " + str.ljust(key+":", max_name_len+2) + simple_fmt(value) for key, value in value.items()])
+            else:
+                value = ", ".join([key + ": " + simple_fmt(value) for key, value in value.items()])
+                need_newline = True
+        else:
+            value = simple_fmt(value)
+
+        return value, need_newline
+
+    def create_mean_records_core(self, new_col_dict, records, mean_cols, group_by):
         values = {}
 
         for col in mean_cols:
@@ -1068,7 +1231,11 @@ class ReportBuilder():
                 err_col_name = col + "-err"
                 mr[err_col_name] = std_err
 
+                new_col_dict[col + "-mean"] = 1
+                new_col_dict[col + "-err"] = 1
+
             mean_records.append(mr)
+            new_col_dict["item_count"] = 1
 
         else:
             for col in mean_cols:
@@ -1085,19 +1252,41 @@ class ReportBuilder():
 
         return mean_records
 
+    def remove_col_prefix(self, name: str):
+        if name.startswith("hparams."):
+            name = name[8:]
+        elif name.startswith("metrics."):
+            name = name[7:]
+        elif name.startswith("tags."):
+            name = name[5:]
+
+        return name
+
     def create_mean_records(self, user_cols, records, mean_cols, group_by, sort_by, reverse, first, last):
 
+        new_col_dict = {}
+
         if group_by:
+            # get user-specified formatting for group_by
+            _, fmt, _ = self.get_user_col_info(user_cols, group_by)
+            if fmt:
+                fmt = "{:" + fmt + "}" 
+
             # calc mean over each groups of records
-            grouped_records = self.group_by(records, group_by)
+            grouped_records = self.group_by(records, group_by, fmt)
             mean_records = []
-            for i, (group, grecords) in enumerate(grouped_records.items()):
-                recs = self.create_mean_records_core(grecords, mean_cols, group_by)
+
+            for i, (group_name, group_records) in enumerate(grouped_records.items()):
+                recs = self.create_mean_records_core(new_col_dict, group_records, mean_cols, group_by)
                 mean_records += recs
 
         else:
             # calc mean over all records
-            mean_records = self.create_mean_records_core(grecords, mean_cols, group_by)
+            mean_records = self.create_mean_records_core(new_col_dict, records, mean_cols, group_by)
+
+        # add new cols to user_cols
+        for nc in new_col_dict:
+            self.add_to_user_columns_if_needed(user_cols, nc)
 
         # apply sort, reverse, first, last
         self.sort_records(mean_records, sort_by, reverse)
@@ -1118,6 +1307,16 @@ class ReportBuilder():
 
         return should
 
+    def fixup_legacy_hp_set_nulls(self, records, exclude_from_hp_set):
+        for record in records:
+            hp_set = utils.safe_value(record, "hp_set", None)
+            if hp_set is None and "hparams" in record:
+
+                arg_dict = record["hparams"]
+                if exclude_from_hp_set:
+                    arg_dict = {key:value for key, value in arg_dict.items() if not key in exclude_from_hp_set}
+                record["hp_set"] = str(arg_dict)
+
     def add_dynamic_cols(self, records, user_cols):
 
         for record in records:
@@ -1265,101 +1464,28 @@ class ReportBuilder():
                 console.diag("build_formatted_table: processing row: {}".format(row_num))
 
             col = None
-            try:
+            catch_exception = False
+
+            if catch_exception:
+                try:
+                    for col_info in col_infos:
+                        first_col, text = self.format_col_to_text(record, col_info, first_col, text, max_fixed_length, 
+                            float_precision, significance, col_space, right_align_num_cols, 
+                            duration_col_names, time_col_names)
+
+                except BaseException as ex:
+                    console.print("Exception formatting col={}: {}".format(col, ex))
+                    console.print("  Exception record: {}".format(record))
+
+                    # debug
+                    raise ex
+
+            else:
+
                 for col_info in col_infos:
-                    if first_col:
-                        first_col = False
-                    else:
-                        text += " " * col_space
-
-                    user_fmt = col_info["user_fmt"] 
-                    if user_fmt == "{:s}":
-                        # don't need this anymore (was used to remove column width restriction)
-                        user_fmt = None
-
-                    col = col_info["name"]
-                    value_type = col_info["value_type"]
-
-                    right_align = right_align_num_cols and (col_info["is_numeric"] or user_fmt or \
-                        col_info["value_type"] == bool) or col in duration_col_names
-
-                    col_width = col_info["col_width"]
-                    align = ">" if right_align else "<"
-
-                    if col == "hp_set":
-                        dummy = 33
-
-                    if not col in record:
-                        # not all records define all columns
-                        str_fmt = "{:" + align + str(col_width)  + "." + str(col_width) + "s}"
-                        text += str_fmt.format("")
-                    else:
-                        value = record[col]
-
-                        # special handling for int values that were logged as floats
-                        # if isinstance(value, float) and value_type is int:
-                        #     value = int(value)
-                        if isinstance(value, str) and value.isnumeric():
-                            # handle cases where int was logged incorrectly as a string
-                            value = int(value)
-
-                        #console.print("col=", col, ", value=", value, ", type(value)=", type(value))
-
-                        # special formatting for time values
-                        if col in duration_col_names:
-                            value = self.format_duration(value, col, record)
-                        elif col in time_col_names:
-                            if isinstance(value, str):
-                                value = arrow.get(value)
-                            value = value.format('YYYY-MM-DD @HH:mm:ss')
-
-                        if user_fmt:
-                            # user provided explict format for this column
-                            if "$" in user_fmt:
-                                # custom XT formatting
-                                value, align = self.xt_custom_format(user_fmt, value)
-                            else:
-                                value = format_value(user_fmt, value)
-
-                            # now treat as string that must fit into col_width
-                            str_fmt = "{:" + align + str(col_width)  + "." + str(col_width) + "s}"
-                            #value = value if value else ""
-                            value = "" if value is None else value
-                            text += format_value(str_fmt, value)
-
-                        elif isinstance(value, float):
-
-                            # default (smart) FLOAT formatting
-                            precision = col_info["precision"] if "precision" in col_info else float_precision
-                            if precision > max_fixed_length:
-                                # use SCIENTIFIC NOTATION
-                                float_fmt = "{:" + align + str(col_width) + "." + str(significance-1) + "e}"
-                            else:
-                                # use FIXED POINT formatting
-                                float_fmt = "{:" + align + str(col_width) + "." + str(precision) + "f}"
-
-                            text += format_value(float_fmt, value)
-
-                        elif isinstance(value, bool):
-                            bool_fmt = "{!r:" + align + str(col_width) + "}"
-                            text += format_value(bool_fmt, value)
-                        elif isinstance(value, int):
-                            int_fmt = "{:" + align + str(col_width) + "d}"
-                            text += format_value(int_fmt, value)
-                        else:
-                            if col == "sku":
-                                dummy = 3
-                            # ensure value is a string
-                            value = "" if value is None else str(value)
-                            str_fmt = "{:" + align + str(col_width)  + "." + str(col_width) + "s}"
-                            text += format_value(str_fmt, value)
-
-            except BaseException as ex:
-                console.print("Exception formatting col={}: {}".format(col, ex))
-                console.print("  Exception record: {}".format(record))
-
-                # debug
-                raise ex
+                    first_col, text = self.format_col_to_text(record, col_info, first_col, text, max_fixed_length, 
+                        float_precision, significance, col_space, right_align_num_cols, 
+                        duration_col_names, time_col_names)
 
             if highlighted:
                 text += pc_utils.NORMAL
@@ -1377,13 +1503,117 @@ class ReportBuilder():
     
         return text, row_count
 
-    def group_by(self, records, group_col):
+    def format_col_to_text(self, record, col_info, first_col, text, max_fixed_length, 
+        float_precision, significance, col_space, right_align_num_cols, 
+        duration_col_names, time_col_names):
+
+        if first_col:
+            first_col = False
+        else:
+            text += " " * col_space
+
+        user_fmt = col_info["user_fmt"] 
+        if user_fmt == "{:s}":
+            # don't need this anymore (was used to remove column width restriction)
+            user_fmt = None
+
+        col = col_info["name"]
+        value_type = col_info["value_type"]
+
+        right_align = right_align_num_cols and (col_info["is_numeric"] or user_fmt or \
+            col_info["value_type"] == bool) or col in duration_col_names
+
+        col_width = col_info["col_width"]
+        align = ">" if right_align else "<"
+
+        if col == "hp_set":
+            dummy = 33
+
+        if not col in record:
+            # not all records define all columns
+            str_fmt = "{:" + align + str(col_width)  + "." + str(col_width) + "s}"
+            text += str_fmt.format("")
+        else:
+            value = record[col]
+
+            # special handling for int values that were logged as floats
+            # if isinstance(value, float) and value_type is int:
+            #     value = int(value)
+            if isinstance(value, str) and value.isnumeric():
+                # handle cases where int was logged incorrectly as a string
+                value = int(value)
+
+            #console.print("col=", col, ", value=", value, ", type(value)=", type(value))
+
+            # special formatting for time values
+            if value is None:
+                value = ""
+            else:
+                if col in duration_col_names:
+                    value = self.format_duration(value, col, record)
+                elif col in time_col_names:
+                    if isinstance(value, str):
+                        value = arrow.get(value)
+                    value = value.format('YYYY-MM-DD @HH:mm:ss')
+
+            if user_fmt:
+                # user provided explict format for this column
+                value, align = self.format_value(user_fmt, value, col_width)
+
+                # now treat as string that must fit into col_width
+                str_fmt = "{:" + align + str(col_width)  + "." + str(col_width) + "s}"
+                #value = value if value else ""
+                value = "" if value is None else value
+                text += safe_format(str_fmt, value, col_width)
+
+            elif isinstance(value, float):
+
+                # default (smart) FLOAT formatting
+                precision = col_info["precision"] if "precision" in col_info else float_precision
+                if precision > max_fixed_length:
+                    # use SCIENTIFIC NOTATION
+                    float_fmt = "{:" + align + str(col_width) + "." + str(significance-1) + "e}"
+                else:
+                    # use FIXED POINT formatting
+                    float_fmt = "{:" + align + str(col_width) + "." + str(precision) + "f}"
+
+                text += safe_format(float_fmt, value, col_width)
+
+            elif isinstance(value, bool):
+                bool_fmt = "{!r:" + align + str(col_width) + "}"
+                text += safe_format(bool_fmt, value, col_width)
+
+            elif isinstance(value, int):
+                int_fmt = "{:" + align + str(col_width) + "d}"
+                text += safe_format(int_fmt, value, col_width)
+            else:
+                if col == "sku":
+                    dummy = 3
+                # ensure value is a string
+                value = "" if value is None else str(value)
+                str_fmt = "{:" + align + str(col_width)  + "." + str(col_width) + "s}"
+                text += safe_format(str_fmt, value, col_width)
+
+        return first_col, text
+
+    def format_value(self, user_fmt, value, col_width):
+        align = "<"
+
+        if user_fmt and "$" in user_fmt:
+            value_str, align = self.xt_custom_format(user_fmt, value)
+        else:
+            value_str = safe_format(user_fmt, value, col_width)
+
+        return value_str, align
+
+    def group_by(self, records, group_col, fmt):
         groups = {}
         for rec in records:
             if not group_col in rec:
                 continue
 
-            group = rec[group_col]
+            group_value = rec[group_col]
+            group, _ = self.format_value(fmt, group_value, 0)
 
             if not group in groups:
                 groups[group] = []
