@@ -21,17 +21,19 @@ from xtlib.console import console
 from xtlib.helpers import file_helper
 from xtlib.helpers.feedbackParts import feedback as fb
 
-run_info_props = {"_id": 1, "app_name": 1, "box_name": 1, "compute": 1, "create_time": 1, "description": 1, 
+run_info_props = {"_id": 1, "app_name": 1, "box_name": 1, "compute": 1, "cmd_line_args": 1, "create_time": 1, "description": 1, 
     "display_name": 1, "exper_name": 1,
     "from_ip": 1, "from_computer_name": 1, "hp_set": 1, "is_child": 1, "is_parent": 1, "is_outer": 1, "job_id": 1, 
     "node_index": 1, "path": 1, "repeat": 1, "script": 1, "run_index": 1, "run_name": 1, "run_num": 1, "run_guid": 1,
     "search_style": 1, "search_type": 1, "service_type": 1, "sku": 1, "username": 1, "ws_name": 1, 
-    "xt_build": 1, "xt_version": 1, "cluster": 1, "vc":1, "service_run_id": 1}
+    "xt_build": 1, "xt_version": 1, "xt_cmd": 1, "cluster": 1, "vc":1, "service_run_id": 1}
 
 run_stats_props = {"status": 1, "start_time": 1, "last_time": 1, "exit_code": 1, "restarts": 1, 
     "end_time": 1, "queue_duration": 1, "run_duration": 1, "end_id": 1, "metric_names": 1,
      "db_retries": 1, "storage_retries": 1}
 
+all_run_props = {**run_info_props, **run_stats_props}
+
 def expand_run_list(store, db, workspace, name_list):
     '''
     args:
@@ -184,7 +186,9 @@ def remove_run_stats(dd: dict):
 
     return run_stats
 
-user_to_actual = {"box": "box_name", "created": "create_time", "child": "is_child", "cluster": "cluster",
+
+
+user_to_actual = {"box": "box_name", "cmd_line_args": "cmd_line_args", "created": "create_time", "child": "is_child", "cluster": "cluster",
     "description": "description", "display_name": "display_name", "experiment": "exper_name", "exit_code": "exit_code", 
     "from_host": "from_computer_name", "from_ip": "from_ip", 
     "guid": "run_guid", "hp_set": "hp_set", "job": "job_id", "last_time": "last_time",
@@ -192,7 +196,7 @@ user_to_actual = {"box": "box_name", "created": "create_time", "child": "is_chil
     "pool": "pool", "repeat": "repeat", "restarts": "restarts", "run": "run_name", "run_index": "run_index", 
     "run_num": "run_num", "script": "script", "search": "search_type", "search_style": "search_style", 
     "service_type": "service_type", "sku": "sku", "started": "start_time", "status": "status", "target": "compute", 
-    "username": "username", "vc": "vc", "workspace": "ws_name", "xt_build": "xt_build", 
+    "username": "username", "vc": "vc", "workspace": "ws_name", "xt_build": "xt_build", "xt_cmd": "xt_cmd",
     "xt_version": "xt_version", "cluster": "cluster", "vc": "vc", "service_run_id": "service_run_id",
 
     # embedded properties
@@ -210,8 +214,9 @@ std_cols_desc = {
     #"app": "the application associated with the run",
     "box": "the name of the box the run executed on",
     "child": "indicates that this run is a child run",
-    "created": "the time when the run was created", 
     "cluster": "the name of the service cluster for the run", 
+    "cmd_line_args": "the script and its arguments used for this run", 
+    "created": "the time when the run was created", 
     "ended": "when the run execution ended",
     "db_retries": "the total number of database retries performed by the runs of this node",
     "description": "the user specified description associated with the run",
@@ -253,6 +258,7 @@ std_cols_desc = {
     "vc": "the name of the virtual cluster for the run",
     "workspace": "the name of the workspace containing the run",
     "xt_build": "the build date of xt that was used to launch the run",
+    "xt_cmd": "the xt command used to initiate this run",
     "xt_version": "the version of xt that was used to launch the run",
     }
 
@@ -553,9 +559,9 @@ def list_runs(store, config, args, compare=False):
         console.print("runs on {}/{}:".format(store_name, workspace))
 
         # print common hyperparameters on top of report
-        if utils.safe_value(args, "common") and builder.hp_set_formatter.hp_common_set:
+        if utils.safe_value(args, "common") and builder.hp_set_formatter.hp_sets_processed:
             console.print("\nCommon hyperparameters:")
-            for hp,value in builder.hp_set_formatter.hp_common_set.items():
+            for hp,value in builder.hp_set_formatter.hp_unchanged_dict.items():
                 console.print("  {}: {}".format(hp, value))
             console.print("")
 
