@@ -57,8 +57,9 @@ class Runner():
         target_args = args["script_args"]
         code_upload = args["code_upload"]
 
-        # user may have wrong slashes for this OS
-        target_file = file_utils.fix_slashes(target_file)
+        # fix slashes to match target OS of compute node
+        target_is_linux = True   
+        target_file = file_utils.fix_slashes(target_file, is_linux=target_is_linux)
 
         if os.path.isabs(target_file):
             errors.syntax_error("path to app file must be specified with a relative path: {}".format(target_file))
@@ -85,7 +86,9 @@ class Runner():
         if target_file == "docker":
             self.is_docker = True
             
-        if not self.is_docker and code_upload and not os.path.exists(target_file):
+        alt_code = args["code"]
+
+        if code_upload and (not self.is_docker) and (not alt_code) and (not os.path.exists(target_file)):
             errors.env_error("script file not found: {}".format(target_file))
 
         ps_path = args["parent_script"]
@@ -343,7 +346,8 @@ class Runner():
                 self.store.start_run_core(workspace, run_name=run_name, exper_name=exper_name, box_name=box_name, app_name=app_name, 
                     username=username, repeat=repeat, pool=pool, job_id=job_id, node_index=node_index, sku=sku,
                     description=description, aggregate_dest=aggregate_dest, path=path, compute=compute, service_type=service_type, 
-                    search_style=search_style, is_parent=is_parent, tag_dict=tag_dict, display_name=display_name)
+                    search_style=search_style, is_parent=is_parent, tag_dict=tag_dict, display_name=display_name, 
+                    cmd_line_args=str(user_cmd_parts), xt_cmd=args["xt_cmd"])
 
             console.diag("create_run: after start_run")
 
@@ -486,7 +490,7 @@ class Runner():
         return script_dir
 
     def build_docker_cmd(self, docker_name:str, target: str, service_type, bootstrap_dir: str, job_secret: str, 
-            setup_name: str, args: List[str]):
+            setup_name: str, env_vars: dict, args: List[str]):
         '''
         This is only called for backends that don't have their own support for running in docker.  Currently, this is
         just the POOL backend (and Azure Batch?).  This function will replace the user's script cmd with:
@@ -538,7 +542,9 @@ class Runner():
         options = "--rm"
 
         # collect env vars 
-        env_vars = {"XT_IN_DOCKER": 1, "XT_USERNAME": pc_utils.get_username()}
+        env_vars["XT_IN_DOCKER"] = 1
+        env_vars["XT_USERNAME"] = pc_utils.get_username()
+        
         scriptor.add_controller_env_vars(env_vars, self.config, job_secret, "node0")
 
         full_image = login_server + "/" + docker_image if login_server else docker_image
@@ -571,12 +577,13 @@ class Runner():
 
         # flags needed by blobfuse usage
         #options += " --cap-add SYS_ADMIN --device /dev/fuse --security-opt apparmor:unconfined"
-        options += " --privileged"
+        if self.backend.get_name() != "singularity":
+            options += " --privileged"
 
-        # append user options
-        other_options = " ".join(args["docker_other_options"])
-        if other_options:
-            options += " " + other_options
+            # append user options
+            other_options = " ".join(args["docker_other_options"])
+            if other_options:
+                options += " " + other_options
 
         # currently, we don't support running in a windows-based container
         docker_is_windows = False
@@ -625,6 +632,29 @@ class Runner():
 
         args["pip_packages"] = new_pip_packages
 
+    def copy_alt_code_to_snapshot(self, alt_code, snapshot_dir, args):
+
+        if job_helper.is_job_id(alt_code):
+            ws_name = args["workspace"]
+            job_id = alt_code
+
+            files = capture.download_before_files(self.store, job_id, ws_name, source_wildcard="before/code/xt_code.zip", 
+                run_name=None, dest_dir=snapshot_dir, log_events=False, silent=True, unzip=False)
+
+        elif store_utils.is_blob_path(alt_code):
+            ws_name = args["workspace"]
+            self.store.download_files_from_workspace(ws_name, alt_code + "/**", snapshot_dir)
+
+        else:
+            # copy files from alt_code to shotshot_dir
+            file_utils.copy_tree(alt_code, snapshot_dir, args)
+
+        # if xt_code.zip file found, unzip it and delete it
+        fn_zip = snapshot_dir + "/xt_code.zip"
+        if os.path.exists(fn_zip):
+            file_helper.unzip_files(fn_zip, snapshot_dir)
+            os.remove(fn_zip)
+
     def snapshot_all_code(self, bootstrap_dir, snapshot_dir, cmd_parts, args):
         '''
         keep code simple (and BEFORE upload fast):
@@ -633,48 +663,154 @@ class Runner():
             - later: if needed, add 2 extra controller files
             - later: zip the whole thing at once & upload 
         '''
+        alt_code = args["code"]
         code_dirs = args["code_dirs"]
         xtlib_capture = args["xtlib_upload"]
         code_omit = args["code_omit"]
         node_script_path = args["node_script_path"]
-        script_dir = None
-
         code_upload = args["code_upload"]
 
         # this figures out the node's path to the script 
         script_dir = self.adjust_script_dir(cmd_parts, node_script_path)
 
-        if code_upload:
-            for i, code_dir in enumerate(code_dirs):
-                # fixup "$scriptdir" relative paths
-                if "$scriptdir" in code_dir:
-                    code_dir = code_dir.replace("$scriptdir", script_dir)
+        if alt_code:
+            self.copy_alt_code_to_snapshot(alt_code, snapshot_dir, args)
 
-                if "==>" in code_dir:
-                    code_dir, dest_dir = code_dir.split("==>")
-                elif "::" in code_dir:
-                    # "::" is legacy version of "==>"
-                    code_dir, dest_dir = code_dir.split("::")
-                else:
-                    dest_dir = "."
-                capture.make_local_snapshot(code_dir, snapshot_dir, dest_dir, code_omit)
         else:
-            script_dir = snapshot_dir
+            if code_upload:
+                for i, code_dir in enumerate(code_dirs):
+                    # fixup "$scriptdir" relative paths
+                    if "$scriptdir" in code_dir:
+                        code_dir = code_dir.replace("$scriptdir", script_dir)
 
-        if xtlib_capture:
-            xtlib_dir = file_utils.get_xtlib_dir()
+                    if "==>" in code_dir:
+                        code_dir, dest_dir = code_dir.split("==>")
+                    elif "::" in code_dir:
+                        # "::" is legacy version of "==>"
+                        code_dir, dest_dir = code_dir.split("::")
+                    else:
+                        dest_dir = "."
+                    capture.make_local_snapshot(code_dir, snapshot_dir, dest_dir, code_omit)
+            else:
+                script_dir = snapshot_dir
 
-            # copy XTLIB directory to "xtlib" subdir of CODE SNAPSHOT
-            dest_dir = snapshot_dir + "/xtlib"
-            file_utils.ensure_dir_deleted(dest_dir)
-            shutil.copytree(xtlib_dir, dest_dir, ignore=shutil.ignore_patterns("demo_files"))
+            if xtlib_capture:
+                xtlib_dir = file_utils.get_xtlib_dir()
+
+                # copy XTLIB directory to "xtlib" subdir of CODE SNAPSHOT
+                dest_dir = snapshot_dir + "/xtlib"
+                file_utils.ensure_dir_deleted(dest_dir)
+                shutil.copytree(xtlib_dir, dest_dir, ignore=shutil.ignore_patterns("demo_files"))
 
         console.diag("after create local snapshot")
         return script_dir
 
+    def send_email_to_approvers(self, request_id, requested_by, description, target, nodes, runs, cmd):
+        team_dict = self.config.get("team")
+        approver_dict = {key: md for key, md in team_dict.items() if utils.safe_value(md, "approver")}
+        approver_usernames = list(approver_dict)
+        approver_contacts = [md["contact"] for md in approver_dict.values()]
+
+        requested_by_contacts = team_dict[requested_by]["contact"]
+
+        # flatten list of lists
+        approver_contacts = [item for sublist in approver_contacts for item in sublist]
+
+        # for now, assume all entries in approver_contacts are valid email names 
+        # TODO: add support for SMS text to phone numbers
+        subject = "job submission request: {}".format(request_id)
+
+        # make body HTML 
+        store_name = self.config.get("store")
+
+        body = ""
+        body += "<b>job submission request:</b> {}".format(request_id)
+        body += "<br><b>store:</b> {}".format(store_name)
+        body += "<br><b>target:</b> {}".format(target)
+        body += "<br><b>nodes:</b> {:,}, <b>runs:</b> {:,}".format(nodes, runs)
+        body += "<br><b>requested by:</b> {}".format(requested_by)
+        body += "<br><b>description:</b> {}".format(description)
+
+        body += "<br><br><b>cmd:</b> {}".format(cmd)
+
+        body += "<br><br><b>To approve:</b> xt approve {}".format(request_id)
+        body += "<br><b>To reject:</b> xt reject {}\n".format(request_id)
+        sent = False
+
+        try:
+            utils.send_email(self.config, approver_contacts, requested_by_contacts, subject, body)
+            sent = True
+        except BaseException as ex:
+            console.print("email send failed: ex: {}".format(ex))
+
+        return approver_usernames, sent
+
+    def create_request(self, ws_name, snapshot_dir, username, target, nodes, runs, args):
+        description = args["description"]
+        if not description:
+            errors.user_error("when creating a request, the reason for the job must be specified with --description")
+
+        # create a new request in database
+        rd = {"status": "requested", "requested_by": username, "description": description, 
+            "target": target, "nodes": nodes, "runs": runs}
+
+        request_id = self.store.create_request(ws_name, rd)
+        console.print("request created: {}".format(request_id))
+
+        code_omit = args["code_omit"]
+        code_zip = args["code_zip"]
+
+        from xtlib.xt_cmds import orig_xt_cmd
+        run_cmd = orig_xt_cmd
+        code_option = " --code=$requests/{}/code ".format(request_id) 
+
+        # replace the "--request" option with code_option
+        index = run_cmd.index("--req")
+        index2 = run_cmd.find(" ", index+5)
+        if index2 > -1:
+            run_cmd = run_cmd[0:index].strip() + code_option + run_cmd[index2:].strip()
+        else:
+            run_cmd = run_cmd[0:index].strip() + code_option
+
+        # update the request record
+        rd["cmd"] = run_cmd
+        self.store.database.update_requests_record(ws_name, request_id, rd)
+
+        # zip up snapshot_dir and copy to storage: workspace/requests/requestNN/code/xt_code.zip
+        remove_prefix_len = len(snapshot_dir)
+        fn_zip, file_count, raw_mb, zipped_mb = capture.zip_before_files(snapshot_dir, code_omit, code_zip, 
+            remove_prefix_len=remove_prefix_len)
+
+        blob_fn = "/{}/requests/{}/code/xt_code.zip".format(ws_name, request_id)
+        self.store.upload_file_to_workspace(ws_name, blob_fn, fn_zip)
+
+        console.print("  code snapshot created: {:,} files (zipped to: {:,.2f} MB, stored: {})".format(file_count, zipped_mb, blob_fn))
+        #console.print("  zipped & written to blob storage: {}".format(blob_fn))
+
+        # send email to approver(s)
+        approvers, sent = self.send_email_to_approvers(request_id, username, description, target, nodes, runs, run_cmd)
+
+        # final message to requesting user
+        if sent:
+            console.print("  request emailed to approvers: {}".format(", ".join(approvers)))
+
     def process_run_command(self, args):
         self.args = args
 
+        result = self.create_job(args)
+        if result:
+            compute, compute_def, workspace, job_id, sweeps_text, cmds, fake_submit, using_hp, is_distributed, \
+                total_run_count, target_file, ps_path, using_aml_hparam, run_specs, repeat_count, service_type, search_style, \
+                runsets, bootstrap_dir, snapshot_dir, script_dir, snapshot_base, job_secret, resume_name, use_controller  = result
+
+            result = self.submit_job(compute, compute_def, workspace, job_id, sweeps_text, cmds, fake_submit, using_hp, is_distributed, \
+                total_run_count, target_file, ps_path, using_aml_hparam, run_specs, repeat_count, service_type, search_style, \
+                runsets, bootstrap_dir, snapshot_dir, script_dir, snapshot_base, job_secret, resume_name, use_controller, args)
+
+        return result
+
+    def create_job(self, args):
+
         # ensure workspace exists
         workspace = args['workspace']
         dry_run = args['dry_run']
@@ -756,6 +892,19 @@ class Runner():
         args["docker_cmd"] = None
         args["docker_is_windows"] = False
 
+        env_vars = args["env_vars"]
+
+        # expand needed args (used by docker command, runner, and batch/pool backends)
+        # TODO: centralize all this env var processing
+        if env_vars:
+            for name, value in env_vars.items():
+                if value.startswith("$$"):
+                    value = os.getenv(value[2:])
+                    env_vars[name] = value
+
+            # update args copy
+            args["env_vars"] = env_vars
+            
         if not self.backend.provides_container_support():
     
             docker_name = args["docker"]
@@ -764,14 +913,24 @@ class Runner():
                 args["docker"] = docker_name
     
             if docker_name and docker_name != "none":
-                self.build_docker_cmd(docker_name, compute, service_type, bootstrap_dir, job_secret, setup_name, args)
+                self.build_docker_cmd(docker_name, compute, service_type, bootstrap_dir, job_secret, setup_name, 
+                    env_vars, args)
                 
         # BUILD CMDS (from static hparam search, user multi cmds, or single user cmd)
         cmds, runsets, total_run_count, repeat_count, run_specs, using_hp, using_aml_hparam, sweeps_text, search_style = \
             self.build_cmds_with_search(service_type, cmd_parts, parent_script, run_script, run_cmd_from_script, use_controller, dry_run, args)
 
         if dry_run:
-            return
+            return None
+
+        request_flag = args["request"]
+        if request_flag:
+            node_count = args["nodes"]
+            if not node_count:
+                node_count = 1
+
+            self.create_request(workspace, snapshot_dir, username, compute, node_count, total_run_count, args)
+            return None
 
         if not using_hp and not using_aml_hparam:
             # zap values of related arguments for better run reports (omitting empty columns)
@@ -803,6 +962,14 @@ class Runner():
             job_id = self.store.create_job(workspace)
         fb.feedback(job_id)
 
+        return compute, compute_def, workspace, job_id, sweeps_text, cmds, fake_submit, using_hp, is_distributed, \
+            total_run_count, target_file, ps_path, using_aml_hparam, run_specs, repeat_count, service_type, search_style, \
+            runsets, bootstrap_dir, snapshot_dir, script_dir, snapshot_base, job_secret, resume_name, use_controller
+
+    def submit_job(self, compute, compute_def, workspace, job_id, sweeps_text, cmds, fake_submit, using_hp, is_distributed, 
+        total_run_count, target_file, ps_path, using_aml_hparam, run_specs, repeat_count, service_type, search_style, 
+        runsets, bootstrap_dir, snapshot_dir, script_dir, snapshot_base, job_secret, resume_name, use_controller, args):
+
         # expand system names in experiment
         experiment = args["experiment"]
         experiment = utils.expand_xt_vars(experiment, job_id=job_id)
@@ -861,7 +1028,7 @@ class Runner():
             adjustment_scripts = self.core.adjust_job_for_direct_run(job_id, job_runs, cmds, using_hp, experiment, service_type, snapshot_dir, 
                 search_style, args=args)
 
-        # add env vars used by both controller and runs
+        # add env vars used by: setup script, controller, and runs
         env_vars = args["env_vars"]
 
         # create a job guid to uniquely identify this job across all XT instances
