@@ -208,6 +208,6 @@ class DbInterface(Interface):
     def create_db_run(self, rd):
         pass
 
-    # API call
-    def set_job_status(self, ws_name, job_id, status):
-        pass
\ No newline at end of file
+    # # API call
+    # def set_job_status(self, ws_name, job_id, status):
+    #     pass
\ No newline at end of file
