@@ -631,7 +631,7 @@ class MongoDB2(implements(DbInterface)):
         self.mongo_with_retries("update_collection_bulk", dbx, \
             lambda: dbx[collection_name].bulk_write(updates) )
 
-    def get_next_job_id(self, ws_name, id_name):
+    def get_next_id_from_workspace(self, ws_name, id_name):
         fd = {"_id": ws_name}
         dbx = self.mongo_client[DEFAULT_DB]
         path = id_name
@@ -1197,7 +1197,7 @@ class MongoDB2(implements(DbInterface)):
             updates["end_time"] = end_time
 
             # add the unique end_id (relative to ws_name)
-            updates["end_id"] = self.get_next_job_id(ws_name, "next_end_id")
+            updates["end_id"] = self.get_next_id_from_workspace(ws_name, "next_end_id")
 
             self.update_run_info(ws_name, run_name, updates, hparams, metrics)
 
