@@ -264,6 +264,13 @@ class StoreV2():
         '''
         return self.helper.get_share_names()
 
+    # use WORKSPACE for these kinds of calls...
+    # def does_blob_exist(self, workspace, blob_path):
+    #     return self.helper.does_blob_exist(workspace, blob_path)
+
+    # def upload_files(self, workspace, blob_path, local_path):
+    #     return self.helper._upload_files(workspace, local_path, None, True)
+
     # ---- WORKSPACE ----
 
     def ensure_workspace_exists(self, ws_name, flag_as_error=True):
@@ -607,7 +614,7 @@ class StoreV2():
 
     def start_child_run(self, ws_name, parent_run_name, exper_name=None, cmd_line_args=None, all_args=None, description=None, 
             from_ip=None, from_host=None, app_name=None, repeat=None, box_name="local", job_id=None, pool=None, 
-            node_index=None, username=None, aggregate_dest=None, path=None, child_name=None, compute=None, 
+            node_index=None, username=None, aggregate_dest=None, path=None, child_name=None, compute=None, xt_cmd=None,
             service_type=None, search_type=None, sku=None, search_style=None, run_index=None, hp_set=None, display_name=None):
         '''
         This is usually called from the XT COMPUTE box, so we CANNOT get from_ip and from_computer_name from the local machine.
@@ -627,7 +634,8 @@ class StoreV2():
         self.start_run_core(ws_name, child_name, exper_name=exper_name, description=description, username=username,
             box_name=box_name, app_name=app_name, repeat=None, is_parent=False, job_id=job_id, pool=pool, node_index=node_index,
             aggregate_dest=aggregate_dest, path=path, aml_run_id=None, compute=compute, service_type=service_type, search_type=search_type,
-            sku=sku, search_style=search_style, is_child=True, run_index=run_index, hp_set=hp_set, display_name=display_name)           
+            sku=sku, search_style=search_style, is_child=True, run_index=run_index, hp_set=hp_set, display_name=display_name, 
+            cmd_line_args=cmd_line_args, xt_cmd=xt_cmd)           
 
         # finally, add a "child_created" record to the parent
         self.log_run_event(ws_name, parent_run_name, "child_created", {"child_name": child_name}, job_id=job_id)
@@ -635,7 +643,8 @@ class StoreV2():
     def start_run_core(self, ws_name, run_name, exper_name=None, description=None, username=None,
             box_name=None, app_name=None, repeat=None, is_parent=False, job_id=None, pool=None, node_index=None,
             aggregate_dest="none", path=None, aml_run_id=None, compute=None, service_type=None, search_type=None,
-            sku=None, search_style=None, is_child=False, run_index=None, tag_dict=None, hp_set=None, display_name=None):
+            sku=None, search_style=None, is_child=False, run_index=None, tag_dict=None, hp_set=None, display_name=None, 
+            cmd_line_args=None, xt_cmd=None):
 
         console.diag("start_run: after create_next_run_directory")
 
@@ -668,6 +677,7 @@ class StoreV2():
             "box_name": box_name, 
             "compute": compute, 
             "create_time": create_time,
+            "cmd_line_args": cmd_line_args,
             "description": description, 
             "display_name": display_name,
             "exper_name": exper_name, 
@@ -694,6 +704,7 @@ class StoreV2():
             "username": username, 
             "ws_name": ws_name, 
             "xt_build": xt_build, 
+            "xt_cmd": xt_cmd, 
             "xt_version": xt_version, 
             }
 
@@ -1152,10 +1163,36 @@ class StoreV2():
         rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=job_id)
         return rf.does_file_exist(run_fn)
 
+    #---- REQUESTS ----
+
+    def create_request(self, ws_name, request_dict):
+        request_num = self.database.get_next_id_from_workspace(ws_name, "next_request_number")
+        request_id = "request{}".format(request_num)
+
+        # create new record in requests
+        request_dict["request_id"] = request_id
+        self.database.create_request(ws_name, request_id, request_dict)
+
+        # create LAST JOB CREATED blob 
+        blob_fn = constants.WORKSPACE_DIR + "/" + constants.WORKSPACE_LAST_REQUEST
+        self.helper._create_blob(ws_name, blob_fn, str(request_id))
+
+        return request_id
+
+    def update_request_status(self, record, new_status, username):
+        ws_name = record["ws_name"]
+        request_id = record["request_id"]
+
+        record["status"] = new_status
+        record["action_by"] = username
+        record["action_time"] = utils.get_arrow_time_str()
+
+        self.database.update_requests_record(ws_name, request_id, record)
+
     #---- JOBS ----
 
     def create_job(self, ws_name):
-        job_num = self.database.get_next_job_id(ws_name, "next_job_number")
+        job_num = self.database.get_next_id_from_workspace(ws_name, "next_job_number")
 
         job_name = "job{}".format(job_num)
 
@@ -1358,6 +1395,8 @@ def build_store_utils_v2():
     # update PROPERTIES of store_utils for version 2 values
     store_utils.DATA_STORE_ROOT = make_share_name("data")
     store_utils.MODELS_STORE_ROOT = make_share_name("models")
+    store_utils.CODE_STORE_ROOT = make_share_name("code")
+    store_utils.REQUESTS_STORE_ROOT = make_share_name("requests")
     store_utils.STORAGE_FORMAT = constants.STORAGE_FORMAT_V2
 
     # update FUNCTIONS of store_utils for version 1 behavior
