@@ -155,6 +155,10 @@ class StoreBlobObjs():
             if not self.provider.does_container_exist(store_utils.MODELS_STORE_ROOT):
                 self.provider.create_container(store_utils.MODELS_STORE_ROOT)
 
+            # create the CODE SHARE if needed
+            if not self.provider.does_container_exist(store_utils.CODE_STORE_ROOT):
+                self.provider.create_container(store_utils.CODE_STORE_ROOT)
+
     # ---- INTERNAL HELPERS ----
     def _check_ws_name(self, ws_name):
         if not self.is_legal_workspace_name(ws_name):
@@ -230,7 +234,7 @@ class StoreBlobObjs():
         if path is None:
             path = ""
 
-        if "**" in wc_target:
+        if wc_target and "**" in wc_target:
             # case 3: return all blobs for target and child directories
             names = list(self.provider.list_blobs(container, path=path, recursive=True))
         elif wc_target and ("*" in wc_target or "?" in wc_target):
@@ -242,6 +246,11 @@ class StoreBlobObjs():
             names = self.provider.list_blobs(container, path=path, recursive=False)
             #console.print("before fnmatch, names=", names)
             names = [name for name in names if fnmatch(name, wc_target)]
+        elif path and wc_target:
+            # just trying to list a single blob
+            full_path = path + "/" + wc_target
+            names = self.provider.list_blobs(container, path=full_path, recursive=False)
+
         else:
             # case 1: wild_base has no wildcards
             # use delimiter-trick to limit listing to target directory only
@@ -336,10 +345,16 @@ class StoreBlobObjs():
         #console.print("ws_name=", ws_name, ", ws_wildcard=", ws_wildcard)
         files_copied = []
 
-        names = self._list_wild_blobs(container, path, wc_target, include_folder_names=True)
+        if not wc_target:
+            # just the single, specified blob
+            names = [path]
+            blob_dir = os.path.dirname(path)
+        else:
+            names = self._list_wild_blobs(container, path, wc_target, include_folder_names=True)
+            blob_dir = path
+
         console.diag("_download_files: names=", names)
 
-        blob_dir = path
         bd_index = 1 + len(blob_dir)   # add for for trailing slash
         #console.print("blob_dir=", blob_dir, ", bd_index=", bd_index)
 
@@ -598,6 +613,9 @@ class StoreBlobObjs():
         names = [name[4:-4] for name in containers if utils.is_share_name(name)]
         return names
 
+    def does_blob_exist(self, container, blob_path):
+        return self.provider.does_blob_exist(container, blob_path)
+
     # ---- WORKSPACES ----
 
     def is_legal_workspace_name(self, ws_name):
@@ -940,6 +958,16 @@ class BlobsObject():
     #     return dir, fn
 
     def _expand_path(self, path, for_dir=False):
+        '''
+        Is this obsolete?
+        Not operating as expected for this call:
+            store.create_workspace_file("ws4", "requests/request3/code/xt_code.zip", ...)
+
+        In the above case, we end up with path changed to:
+            '__ws__//requests/request14/code/xt_code.zip'
+
+        We expected for path to be left alone in this case.
+        '''
         base_path = self.base_path
 
         if base_path:
@@ -971,8 +999,10 @@ class BlobsObject():
         return self.store.provider.create_blob_from_path(container, path, source_fn, progress_callback=progress_callback)
 
     def upload_files(self, folder, source_wildcard, recursive=False, exclude_dirs_and_files=[], log_files=False):
-        path = self._expand_path(folder)
-        return self.store._upload_files(self.container, path, source_wildcard, recursive=recursive, 
+        #path = self._expand_path(folder)
+        container, path, wc_target = self._get_container_path_target(folder)
+
+        return self.store._upload_files(container, path, source_wildcard, recursive=recursive, 
             exclude_dirs_and_files=exclude_dirs_and_files, log_files=log_files)
 
     def _process_parent_path(self, container, base_path, path):
@@ -1017,6 +1047,20 @@ class BlobsObject():
                 # expand $models
                 path = path.replace("$models", "/" + store_utils.MODELS_STORE_ROOT)
 
+            elif path.startswith("$code"):
+                # expand $models
+                path = path.replace("$code", "/" + store_utils.CODE_STORE_ROOT)
+
+            elif path.startswith("$workspace"):
+                # expand $requests to: /workspace
+                ws_prefix = "/{}".format(self.container)
+                path = path.replace("$workspace", ws_prefix)
+
+            elif path.startswith("$requests"):
+                # expand $requests to: /workspace/requests
+                req_prefix = "/{}/requests".format(self.container)
+                path = path.replace("$requests", req_prefix)
+
             elif path.startswith("$jobs"):
                 # expand $jobs to: /workspace/jobs
                 job_prefix = "/{}/jobs".format(self.container)
@@ -1064,6 +1108,9 @@ class BlobsObject():
                 errors.syntax_error("wildcard characters are not allowed in directory part of store paths")
         else:
             wc_target = None
+            # # treat filename at base like a wildcard spec
+            # wc_target = os.path.basename(path)
+            # path = os.path.dirname(path)
 
         #console.print("container=", container, ", path=", path, ", wc_target=", wc_target)
         return container, path, wc_target
