@@ -222,7 +222,7 @@ class StoreBlobObjs():
 
         if "**" in wc_target:
             # case 3: return all blobs for target and child directories
-            names = list(self.provider.list_blobs(container, path=path))
+            names = list(self.provider.list_blobs(container, path=path, recursive=True))
         elif wc_target and ("*" in wc_target or "?" in wc_target):
             # case 2: normal wildcards specified (for target directory only)
             # use delimiter-trick to limit listing to target directory only
@@ -391,7 +391,7 @@ class StoreBlobObjs():
                 for ws_name in folder_names:
                     # get blobs from AZURE
                     console.diag("reading blobs for ws={}".format(ws_name))
-                    blobs  = self.provider.list_blobs(ws_name, path=None, return_names=False)
+                    blobs  = self.provider.list_blobs(ws_name, path=None, return_names=False, recursive=True)
                     blobs = list(blobs)
 
                     ws_folders = self._build_folders_from_blobs(blobs, ws_name, 
@@ -401,7 +401,7 @@ class StoreBlobObjs():
         else:
             # get blobs from AZURE
             actual_path = path if path else None
-            blobs  = self.provider.list_blobs(container, path=actual_path, return_names=False)
+            blobs  = self.provider.list_blobs(container, path=actual_path, return_names=False, recursive=True)
             blobs = list(blobs)
 
             if wc_target:
@@ -519,7 +519,8 @@ class StoreBlobObjs():
             rel_path = path[base_name_len:]
             #console.print("rel_path=", rel_path)
 
-            folder = {"folder_name": display_name}
+            folder = {"name": display_name}
+            folder["full_path"] = display_name
             folder["dirs"] = []
             folder["files"] = []
             folder["level"] = 1 + rel_path.count("/") if rel_path else 0
@@ -534,7 +535,8 @@ class StoreBlobObjs():
     def _get_root_folders(self):
         folder_names = self.provider.list_containers()
 
-        folder = {"folder_name": "/"}
+        folder = {"name": "/"}
+        folder["full_path"] = "/"
         folder["level"] = 0
         folder["files"] = []
         folder["dirs"] = folder_names
@@ -556,8 +558,8 @@ class StoreBlobObjs():
 
         return old_count
         
-    def list_blobs(self, container, blob_path, return_names=True):
-        blobs  = self.provider.list_blobs(container, path=blob_path, return_names=return_names)
+    def list_blobs(self, container, blob_path, return_names=True, recursive=True):
+        blobs  = self.provider.list_blobs(container, path=blob_path, return_names=return_names, recursive=recursive)
         return blobs
 
     # ---- SHARES ----
@@ -730,7 +732,7 @@ class StoreBlobObjs():
         # so, we need to enumerate and delete each one
         
         # since we are deleting from the list we are enumerating, safest to grab the whole list up frong
-        blobs = list(self.provider.list_blobs(ws_name, path=self._run_path(run_name) + "/"))
+        blobs = list(self.provider.list_blobs(ws_name, path=self._run_path(run_name) + "/"), recursive=True)
         for blob in blobs:
             self.provider.delete_blob(ws_name, blob)   
 
@@ -740,7 +742,7 @@ class StoreBlobObjs():
 
         # copy a single blob at a time
         #for source_blob_path in self.bs.list_blob_names(source_workspace_name, source_run_name):
-        for source_blob in self.provider.list_blobs(source_workspace_name, path=self._run_path(source_run_name)  + "/"):
+        for source_blob in self.provider.list_blobs(source_workspace_name, path=self._run_path(source_run_name)  + "/", recursive=True):
             dest_blob_path = self._run_path(dest_run_name)  + "/" + self._remove_first_node(source_blob)
 
             # copy single blob within same storage service
