@@ -44,7 +44,12 @@ def simplify_records_id(records):
         for r in records:
             simplify_id(r)
 
+def is_blob_path(path):
+   return path.startswith("$") or path.startswith("blob-store://")
+
 MODELS_STORE_ROOT = None
+CODE_STORE_ROOT = None
+REQUESTS_STORE_ROOT = None
 DATA_STORE_ROOT = None
 STORAGE_FORMAT = None
 
