@@ -893,3 +893,55 @@ def debug_path(path):
 
         path = os.path.dirname(path)
 
+def send_email_via_outlook(to_list, cc_list, subject, body):
+    '''
+    Pro's:
+        - quick to send
+        - recognized sender
+        - shows up an internal (not flagged EXTERNAL)
+        
+    Con's:
+        - requires user to have Outlook installed
+        - user cannot launch XT from admin command window
+    '''
+    import win32com.client as win32
+
+    outlook = win32.Dispatch('outlook.application')
+    mail = outlook.CreateItem(0)
+
+    mail.To = ";".join(to_list)
+    mail.Cc = ";".join(cc_list)
+    mail.Subject = subject
+    mail.Body = body
+    mail.HTMLBody = body
+
+    # To attach a file to the email (optional):
+    # attachment  = "Path to the attachment"
+    # mail.Attachments.Add(attachment)
+
+    mail.Send()
+
+def send_email(config, to_list, cc_list, subject, body):
+    '''
+    Pro's:
+        - no special client requirements
+    Con's:
+        - sender domain is ugly 
+        - flagged as EXTERNAL by Outlook 
+    '''    
+    from azure.communication.email import EmailClient, EmailContent, EmailAddress, EmailMessage, EmailRecipients
+
+    email_cs = config.get("external-services", "xtnotification", "connection-string")
+    email_from = config.get("external-services", "xtnotification", "from")
+
+    client = EmailClient.from_connection_string(email_cs)
+    content = EmailContent(subject=subject, plain_text=body, html=body)
+
+    to_addr_list = [EmailAddress(email=to) for to in to_list]
+    cc_addr_list = [EmailAddress(email=cc) for cc in cc_list]
+
+    message = EmailMessage(sender=email_from, content=content,
+        recipients=EmailRecipients(to=to_addr_list, cc=cc_addr_list))
+
+    response = client.send(message)
+    return response
