@@ -25,6 +25,9 @@ from xtlib.impl_shared import ImplShared
 
 logger = logging.getLogger(__name__)
 
+# global 
+orig_xt_cmd = None
+
 def get_fn_local_config(args):
     '''
     gets path to local config file (specified on cmd line or defaulting to ./xt_config.yaml)
@@ -119,7 +122,10 @@ def main(cmd=None, new_start_time=None, capture_output=False, basic_mode=False,
         global xt_start_time
         xt_start_time = new_start_time
 
+    global orig_xt_cmd
+
     if cmd:
+        orig_xt_cmd = cmd
         cmd = cmd.strip()
 
         if cmd.startswith("xt "):
@@ -133,6 +139,7 @@ def main(cmd=None, new_start_time=None, capture_output=False, basic_mode=False,
         args = [arg for arg in args if arg]
     else:
         # if caller did not supply cmd
+        orig_xt_cmd = " ".join(sys.argv)
         args = sys.argv[1:]
    
     # when executing multiple commands, reset the feedback for each command
