@@ -78,8 +78,17 @@ def main(cmd=None, disable_quickstart = False, capture_output=False, mini=False)
         # treat as if it came from the shell (for consistent debugging/support)
         console.diag("orig cmd={}".format(cmd))
 
-        # shlex and linux loses single quotes around strings, but windows does not
+        # shlex and LINUX SHELL lose single quotes around strings, but windows does not
+        # shlex also zaps single backslashes
+        cmd = cmd.replace("\\", "\\\\")    # double backslashes to prevent their removal
+
         orig_args = shlex.split(cmd)
+
+        # now, undouble backslashes
+        for a, arg in enumerate(orig_args):
+            if "\\\\" in arg:
+                orig_args[a] = arg.replace("\\\\", "\\")
+
         console.diag("shlex args={}".format(orig_args))
     else:
         orig_args = sys.argv[1:]
